<?php

namespace Frisbo\WooCommerce;

/**
 * Frisbo main class
 */

use Frisbo\FrisboSdk\Caches\FrisboFileCache as FrisboCache;
use Twig\Loader\FilesystemLoader;
use Twig\Environment;

final class Frisbo
{
    /**
     * Frisbo version.
     *
     * @var string
     */
    public $version = '2.1.9';

    /**
     * Frisbo plugin identifier.
     *
     * @var string
     */
    public $pluginName = 'frisbo-woocommerce/frisbo-woocommerce.php';

    /**
     * Woocommerce plugin identifier.
     *
     * @var string
     */
    public $wooPluginName = 'woocommerce/woocommerce.php';

    /**
     * The single instance of the class.
     *
     * @var Frisbo
     */
    private static $instance = null;

    /**
     * The templating engine
     *
     * @var Environment
     */
    public $twig;

    /**
     * The frisbo plugin settings
     *
     * @var FrisboCache
     */
    public $cache;

    /**
     * Used to create requests to the Frisbo API
     *
     * @var FrisboWordpressClient
     */
    public $client;

    /**
     * The frisbo plugin settings
     *
     * @var FrisboPluginSettings
     */
    public $settings;

    /**
     * The frisbo logger
     *
     * @var FrisboLogger
     */
    public $logger;

    /**
     * @var string
     */
    const SELLING_CHANNEL_PREFIX = 'WOO';

    /**
     * @var int
     */
    const TIMEOUT = 60;

    /**
     * Frisbo main instance.
     * Ensures only one instance of Frisbo is loaded or can be loaded.
     */
    public static function instance(): self
    {
        if (null === self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * Frisbo Constructor.
     */
    private function __construct()
    {
        $this->constants();
        $this->includes();
        $this->initHooks();
        $this->scheduleCronEvents();

        $this->twig = new Environment(new FilesystemLoader(FRISBO_VIEWS));
        $this->settings = new FrisboPluginSettings();

        if (!is_dir(FRISBO_TMP)) {
            mkdir(FRISBO_TMP);
        }

        $this->logger = new FrisboLogger;
        $this->cache = new FrisboCache(FrisboCache::CACHE_PREFIX, ['CACHE_FOLDER' => FRISBO_TMP]);

        if ($this->settings->email && $this->settings->password) {
            $this->setClient();
            $this->client->setLogger($this->logger);
        }
        include_once FRISBO_PATH . '/app/FrisboAdmin.php';
    }

    /**
     * Include required core files
     */
    public function includes()
    {
        include_once FRISBO_PATH . '/app/FrisboEnvs.php';
        include_once FRISBO_PATH . '/app/FrisboDebugger.php';
        include_once FRISBO_PATH . '/app/FrisboTrait.php';
        include_once FRISBO_PATH . '/app/FrisboLogger.php';
        include_once FRISBO_PATH . '/app/FrisboAdmin.php';
        include_once FRISBO_PATH . '/app/FrisboPluginSettings.php';
        include_once FRISBO_PATH . '/app/FrisboProducts.php';
        include_once FRISBO_PATH . '/app/FrisboOrders.php';
    }

    public function hasLoggedIn(): bool
    {
        return !empty($this->settings->email) && !empty($this->settings->password) ? true : false;
    }

    public function refreshSettings()
    {
        $this->settings = new FrisboPluginSettings();
    }

    public function setDefaultSettings()
    {
        $this->settings = new FrisboPluginSettings(true);
        $this->settings->updateSettings();
    }

    public function setClient(string $email = null, string $password = null)
    {
        $this->client =  new FrisboWordpressClient([
            'username' => $email ?? $this->settings->email,
            'password' => $password ?? $this->settings->password
        ], $this->cache);
    }

    /**
     * Define Frisbo Constants.
     */
    private function constants()
    {
        if (!defined('FRISBO_PATH')) {
            define('FRISBO_PATH', dirname(FRISBO_PLUGIN_FILE));
        }
        if (!defined('FRISBO_URL')) {
            define('FRISBO_URL', plugin_dir_url(FRISBO_PLUGIN_FILE));
        }
        if (!defined('FRISBO_VIEWS')) {
            define('FRISBO_VIEWS', FRISBO_PATH . '/views');
        }
        if (!defined('FRISBO_TMP')) {
            define('FRISBO_TMP', FRISBO_PATH . '/tmp/');
        }
        if (!defined('FRISBO_TESTS')) {
            define('FRISBO_TESTS', FRISBO_PATH . '/tests');
        }
    }

    /**
     * Hook into actions and filters.
     *
     */
    private function initHooks()
    {
        \add_filter('cron_schedules',  [$this, 'cron_schedules']);
    }

    /**
     * Schedule events
     *
     */
    private function scheduleCronEvents()
    {
        if (!\wp_next_scheduled('frisbo_sync_products_cron')) {
            \wp_schedule_event(time(), 'every_fifteen_minutes', 'frisbo_sync_products_cron');
        }
        if (!\wp_next_scheduled('frisbo_sync_products_stock_cron')) {
            \wp_schedule_event(time(), 'every_minute', 'frisbo_sync_products_stock_cron');
        }
        if (!\wp_next_scheduled('frisbo_sync_last_products_stock_cron')) {
            \wp_schedule_event(time(), 'every_two_minutes', 'frisbo_sync_last_products_stock_cron');
        }
        if (!\wp_next_scheduled('frisbo_sync_orders_cron')) {
            \wp_schedule_event(time(), 'every_two_minutes', 'frisbo_sync_orders_cron');
        }
        if (!\wp_next_scheduled('frisbo_sync_orders_status_cron')) {
            \wp_schedule_event(time(), 'every_ten_minutes', 'frisbo_sync_orders_status_cron');
        }
    }

    /**
     * Add cron schedules.
     *
     * @param array $schedules List of WP scheduled cron jobs.
     */
    public function cron_schedules(array $schedules): array
    {
        $schedules['every_minute'] = array(
            'interval'  => \MINUTE_IN_SECONDS,
            'display'   => \__('Every minute', 'frisbo')
        );
        $schedules['every_two_minutes'] = array(
            'interval'  => 2 * \MINUTE_IN_SECONDS,
            'display'   => \__('Every 2 Minutes', 'frisbo')
        );
        $schedules['every_ten_minutes'] = array(
            'interval'  => 10 * \MINUTE_IN_SECONDS,
            'display'   => \__('Every ten minutes', 'frisbo')
        );
        $schedules['every_fifteen_minutes']     = [
            'interval' =>  15 * \MINUTE_IN_SECONDS,
            'display'  => \__('Every 15 Minutes', 'frisbo'),
        ];
        return $schedules;
    }
}
