<?php

namespace Frisbo\WooCommerce;

use Exception;
use Frisbo\FrisboSdk\FrisboCache;

class FrisboAdmin
{
    /**
     * @var string
     */
    const NONCE = 'frisbo_admin';

    private $frisbo;

    public function __construct()
    {
        $this->frisbo = Frisbo::instance();

        \add_action('wp_ajax_frisbo_login', [$this, 'login']);
        \add_action('wp_ajax_frisbo_update_plugin_settings', [$this, 'updatePluginSettings']);
        \add_action('wp_ajax_frisbo_logout', [$this, 'logout']);
        \add_action('admin_enqueue_scripts', [$this, 'adminScripts']);
        \add_action('admin_menu', array($this, 'adminMenu'));
    }

    /**
     * Adds the Frisbo label to the WordPress Admin Sidebar Menu
     */
    public function adminMenu()
    {
        \add_menu_page(
            'Frisbo',
            'Frisbo',
            'manage_options',
            'frisbo',
            array($this, 'adminLayout'),
            FRISBO_URL . 'assets/img/frisbo-favicon.ico',
            56.4
        );
    }


    /**
     * Outputs the Admin Dashboard layout containing the form with all its options
     *
     * @return void
     */
    public function adminLayout()
    {
        if (!\is_plugin_active($this->frisbo->pluginName)) {
            return;
        }

        if (!\is_plugin_active($this->frisbo->wooPluginName)) {
            echo $this->frisbo->twig->render('error.twig', [
                'message' => 'Frisbo plugin requires Woocommerce installed.'
            ]);
            return;
        }

        if ($this->frisbo->hasLoggedIn()) {
            /**
             * TODO: to be removed. 
             * These are new additions to the plugin settings
             * and will cause issues if not existing.   
             */
            $this->setOrganizations();
            $this->setChannels();
            echo $this->frisbo->twig->render('admin.twig', [
                'version' => $this->frisbo->version,
                'settings' => $this->frisbo->settings->getSettings(),
                'tagline' => \__('Configure the integration settings bellow.', 'frisbo'),
                'tooltip' => $this->getTooltips(),
            ]);
            return;
        }
        echo $this->frisbo->twig->render('login.twig', [
            'settings' => $this->frisbo->settings->getSettings(),
            'tagline' => \__('One step closer to smart fulfillment. Enter your Frisbo credentials or', 'frisbo'),
            'callToAction' => \__('request a demo', 'frisbo'),
            'callToActionUrl' => 'https://www.frisbo.eu/#requestADemo',
        ]);
    }

    private function getTooltips()
    {
        return [
            'productsAmount' => \__('Number of products to be synchronized per run. Useful to tune if you have a slow server. We recommend leaving the default values.', 'frisbo'),
            'stockAmount' => \__('Number of products to have their stock synchronized per query. Useful to tune if you have a slow server. We recommend leaving the default values.', 'frisbo'),
            'ordersAmount' => \__('Number of orders to be synchronized per run. Useful to tune if you have a slow server. We recommend leaving the default values.', 'frisbo'),
            'orderPrefix' => \__('e.g. _PREFIX', 'frisbo'),
            'autoFulfillment' => \__('Automatically synchronize products and orders with Frisbo. This will also update the order status when it\'s shipped.', 'frisbo'),
            'overwriteStock' => \__('Automatically update products stock and overwrite current product quantity.', 'frisbo'),
            'noStock' => \__('If unchecked, we\'ll not synchronize stocks that we haven\'t received (we will not overwrite them with 0). If checked, products having no stock will have the quantity set to 0.', 'frisbo'),
            'useBillingForShipping' => \__('We\'ll use the billing customer on the shipping customer data for mentioned payment methods. This may be requested for the courier service invoice.', 'frisbo'),
            'markOrderAsManual' => \__('Mark order as manual based on the payment method', 'frisbo'),
            'orderStatusSync' => \__('Synchronize the orders in a different status than the default ones ( processing and completed ). It can be a comma separated list, and you may also specify the payment method as follows {payment_method}:{order_status}. If no payment method is specified we’ll update the default synchozitation statuses.', 'frisbo'),
            'eanFields' => \__('Add the product EAN field name in order to be sent to Frisbo when synchronizing the products. It can be a comma separated list, the first ean field found with the identifier will be placed upon the product. In order for this to work the field needs to be in the product meta parameters and can be retrieved using $product->get_meta(\'ean_field_name\') (this is an example)', 'frisbo'),
            'vatRegNumberFields' => \__('Add the billing customer VAT field name (trade registration number) in order to be sent to Frisbo when synchronizing the orders. It can be a comma separated list, the first VAT field found with the identifier will be placed upon the billing customer. For this to work the field needs to be in the order meta parameters and can be retrieved using $order->get_meta(\'vat_field_name\') (this is an example).', 'frisbo'),
            'lockerFields' => \__('Add the Locker ID field name in order to be sent to Frisbo when synchronizing the orders. It can be a comma separated list, and it needs to be as follows: {courier_name}@{locker_id_field_name} e.g. DHL@_dhl_locker_id. For this to work the field needs to be in the order meta parameters and can be retrieved using $order->get_meta(\'locker_id_field_name\') (this is an example).', 'frisbo'),
            'currencyFields' => \__('Add the Currency field name in order to be sent to Frisbo when synchronizing the orders. The value of this field will be shown on the invoice. It can be a comma separated list, the first currency field found will be placed on the order. For this to work the field needs to be in the order meta parameters and can be retrieved using $order->get_meta(\'curency_field_name\') (this is an example).', 'frisbo'),
            'packingNoteFields' => \__('Add the packing notes field name in order to be sent to Frisbo when synchronizing the orders. It can be a comma separated list, the first packing note field found will be placed on the order. For this to work the field needs to be in the order meta parameters and can be retrieved using $order->get_meta(\'packing_note_field_name\') (this is an example).', 'frisbo'),
            'shippingNameMapping' => \__('Map a shipping name to a courier such that we send the order to Frisbo with the correct courier name.', 'frisbo'),
            'moveOrderToStatus' => \__('When an order is shipped in Frisbo we\'ll update the Woocommerce order status using the value of this setting.', 'frisbo'),
            'moveCanceledOrderToStatus' => \__('When an order is canceled in Frisbo we\'ll update the Woocommerce order status using the value of this setting. "-" may be used to not change the order status.', 'frisbo'),
            'moveReturnedOrderToStatus' => \__('When an order is returned in Frisbo we\'ll update the Woocommerce order status using the value of this setting. "-" may be used to not change the order status.', 'frisbo'),
            'addressShippingFields' => \__('Add your shipping address fields here. You can separate them by ` , `. Their values will be concatenated and added to the order shipping address field. Ex: __shipping_address, _shipping_street_number', 'frisbo'),
            'addressBillingFields' => \__('Add your billing address fields here. You can separate them by ` , `. Their values will be concatenated and added to the order billing address field. Ex: __billing_address, _billing_street_number', 'frisbo'),
        ];
    }

    /**
     * Adds Admin Scripts for the Ajax call
     */
    public function adminScripts()
    {
        \wp_enqueue_script('frisbo-admin', FRISBO_URL . 'assets/js/admin.js', array(), 1.1);
        $admin_options = array(
            'ajax_url' => \admin_url('admin-ajax.php'),
            '_nonce'   => \wp_create_nonce(self::NONCE),
        );
        \wp_localize_script('frisbo-admin', 'frisbo_exchanger', $admin_options);
        \wp_enqueue_style('frisbo-style', FRISBO_URL . 'assets/css/frisbo.css');
    }

    public function logout()
    {
        $this->frisbo->settings->email = '';
        $this->frisbo->settings->password = '';
        $this->frisbo->settings->updateSettings();
        Frisbo::instance()->cache->forget(FrisboCache::FRISBO_TOKEN_KEY);
    }

    public function login()
    {
        $email = $this->sanitize($_POST['email']);
        $password = $this->sanitize($_POST['password']);
        $this->frisbo->setClient($email, $password);
        try {
            $this->frisbo->client->login($email, $password);
            if ($email && $this->frisbo->settings->email != $email) {
                $this->frisbo->setDefaultSettings();
                $this->logout();
            }
            $this->frisbo->settings->email = $email;
            $this->frisbo->settings->password = $password;
            $this->frisbo->settings->updateSettings();
        } catch (Exception $ex) {
            \wp_send_json_error($ex->getMessage());
        }

        $this->setOrganizationId(true);
        $this->setChannelId(true);
    }


    public function updatePluginSettings()
    {
        $organizationId = $this->sanitize($_POST['organizationId'], 'integer');
        if (empty($organizationId)) {
            $this->setOrganizationId(true);
        }

        $changedOrg = false;
        if (!empty($organizationId) && $this->isDifferentOrganization($organizationId)) {
            $this->frisbo->settings->organizationId = $organizationId;
            $this->setChannelId(true);
            $changedOrg = true;
        }

        $channelId = $this->sanitize($_POST['channelId'], 'integer');
        if (!empty($channelId) && !$changedOrg) {
            $this->frisbo->settings->channelId = $channelId;
        }

        $this->frisbo->settings->autoFulfillment = $this->sanitize($_POST['autoFulfillment'], 'checkbox');
        $this->frisbo->settings->overwriteStock =  $this->sanitize($_POST['overwriteStock'], 'checkbox');
        $this->frisbo->settings->noStock =  $this->sanitize($_POST['noStock'], 'checkbox');
        $this->frisbo->settings->useBillingForShipping =  $this->sanitize($_POST['useBillingForShipping']);
        $this->frisbo->settings->productsAmount =  $this->sanitize($_POST['productsAmount'], 'integer');
        $this->frisbo->settings->stockAmount =  $this->sanitize($_POST['stockAmount'], 'integer');
        $this->frisbo->settings->ordersAmount =  $this->sanitize($_POST['ordersAmount'], 'integer');
        $this->frisbo->settings->orderPrefix =  $this->sanitize($_POST['orderPrefix']);
        $this->frisbo->settings->orderStatusSync =  $this->sanitize($_POST['orderStatusSync']);
        $this->frisbo->settings->moveOrderToStatus =  $this->sanitize($_POST['moveOrderToStatus']);
        $this->frisbo->settings->markOrderAsManual =  $this->sanitize($_POST['markOrderAsManual']);
        $this->frisbo->settings->moveCanceledOrderToStatus =  $this->sanitize($_POST['moveCanceledOrderToStatus']);
        $this->frisbo->settings->moveReturnedOrderToStatus =  $this->sanitize($_POST['moveReturnedOrderToStatus']);
        $this->frisbo->settings->eanFields =  $this->sanitize($_POST['eanFields']);
        $this->frisbo->settings->vatRegNumberFields =  $this->sanitize($_POST['vatRegNumberFields']);
        $this->frisbo->settings->lockerFields =  $this->sanitize($_POST['lockerFields']);
        $this->frisbo->settings->currencyFields =  $this->sanitize($_POST['currencyFields']);
        $this->frisbo->settings->packingNoteFields =  $this->sanitize($_POST['packingNoteFields']);
        $this->frisbo->settings->shippingNameMapping =  $this->sanitize($_POST['shippingNameMapping']);
        $this->frisbo->settings->addressShippingFields =  $this->sanitize($_POST['addressShippingFields']);
        $this->frisbo->settings->addressBillingFields =  $this->sanitize($_POST['addressBillingFields']);
        $this->frisbo->settings->updateSettings();
    }

    private function isDifferentOrganization($newOrganizationId)
    {
        if ($newOrganizationId != $this->frisbo->settings->organizationId) {
            return true;
        }
        return false;
    }

    private function setOrganizationId($force = false)
    {
        if ($this->frisbo->settings->organizations && !$force) {
            return;
        }
        try {
            $this->frisbo->settings->organizations = $this->frisbo->client->getOrganizations();
            $this->frisbo->settings->organizationId = $this->frisbo->settings->organizations[0]->organization_id;
            $this->frisbo->settings->updateSettings();
        } catch (\Exception $ex) {
            $this->frisbo->logger->info('FrisboClient::' . $ex->getMessage());
        }
    }

    private function setChannelId($force = false)
    {
        if ($this->frisbo->settings->channels && !$force) {
            return;
        }
        try {
            $this->frisbo->settings->channels = $this->frisbo->client->getOrganizationChannels($this->frisbo->settings->organizationId);
            $this->frisbo->settings->channelId = $this->frisbo->settings->channels[0]->id;
            $this->frisbo->settings->updateSettings();
        } catch (\Exception $ex) {
            $this->frisbo->logger->info('FrisboClient::' . $ex->getMessage());
        }
    }

    private function setOrganizations()
    {
        if ($this->frisbo->settings->organizations) {
            return;
        }
        try {
            $this->frisbo->settings->organizations = $this->frisbo->client->getOrganizations();
            $this->frisbo->settings->updateSettings();
        } catch (\Exception $ex) {
            $this->frisbo->logger->info('FrisboClient::' . $ex->getMessage());
        }
    }

    private function setChannels()
    {
        if ($this->frisbo->settings->channels || !$this->frisbo->settings->organizationId) {
            return;
        }
        try {
            $this->frisbo->settings->channels = $this->frisbo->client->getOrganizationChannels($this->frisbo->settings->organizationId);
            $this->frisbo->settings->updateSettings();
        } catch (\Exception $ex) {
            $this->frisbo->logger->info('FrisboClient::' . $ex->getMessage());
        }
    }

    private function sanitize($setting, $type = 'text')
    {
        $setting = \sanitize_text_field($setting);
        if ($type == 'checkbox') {
            return !empty($setting) ? 'on' : 'off';
        }
        if ($type == 'integer') {
            return is_numeric($setting) && $setting > 0 ? $setting : null;
        }
        return $setting;
    }
}

new FrisboAdmin();
