<?php

namespace Frisbo\WooCommerce;

use Frisbo\FrisboSdk\Models\Order;
use Frisbo\FrisboSdk\Models\OrderAddress;
use Frisbo\FrisboSdk\Models\OrderCustomer;
use Frisbo\FrisboSdk\Models\OrderProduct;

class FrisboOrderDto
{
    use FrisboTrait;

    private $frisbo;
    private $order;

    public function __construct(\WC_Order $order)
    {
        $this->order = $order;
        $this->frisbo = Frisbo::instance();
    }

    public function getMappings(): Order
    {
        $paymentMethod = $this->order->get_payment_method();
        $notes = [];
        if (!empty($this->order->get_customer_note())) {
            $notes[] = $this->order->get_customer_note();
        }
        if (!empty($paymentMethod)) {
            $notes[] = \__('Payment method:', 'frisbo') . ' ' . $paymentMethod;
        }

        $order = new Order();
        $order->channel_id = $this->frisbo->settings->channelId;
        $order->organization_id = $this->frisbo->settings->organizationId;
        $order->warehouse_id = -1;
        $order->ordered_date = \get_gmt_from_date($this->order->get_date_created()->date('Y-m-d H:i:s'));
        $order->order_reference = $this->getOrderReference($this->order);
        $order->notes = implode(' / ', $notes);
        $order->packing_notes = $this->getPackingNotes($this->order);
        $order->transport_tax = 0;
        $order->cash_on_delivery = $paymentMethod == 'cod' ? true : false;
        $order->is_manual = $this->isManualOrder($this->order);
        $order->discount = $this->order->get_discount_total() + $this->order->get_discount_tax();

        $order->products = $this->getOrderProducts($this->order);
        $order->metadata = $this->getOrderMetadata($this->order);

        $order->shipping_customer = $this->getShippingCustomer($this->order);
        $order->billing_customer = $this->getBillingCustomer($this->order);
        $order->shipping_address = $this->getShippingAddress($this->order);
        $order->billing_address = $this->getBillingAddress($this->order);

        return $order;
    }

    private function isManualOrder(\WC_Order $order)
    {
        $paymentMethods = explode(',', $this->frisbo->settings->markOrderAsManual);
        if ($paymentMethods[0] == '-') {
            return false;
        }
        $orderPaymentMethod = $order->get_payment_method();
        foreach ($paymentMethods as $paymentMethod) {
            if ($orderPaymentMethod == $paymentMethod) {
                return true;
            }
        }
        return false;
    }

    private function getBillingAddress(\WC_Order $order): OrderAddress
    {
        $address = new OrderAddress();
        $address->street = implode(', ', $this->getStreet($order, false));
        $address->city = $order->get_billing_city();
        $address->county = $this->getBillingCounty($this->order);
        $address->country = $this->getBillingCountry($this->order);
        $address->zip = $order->get_billing_postcode();
        return $address;
    }

    private function getShippingAddress(\WC_Order $order): OrderAddress
    {
        $address = new OrderAddress();
        $address->street = implode(', ', $this->getStreet($order));
        $address->city = $this->getShippingCity($this->order);
        $address->county = $this->getShippingCounty($this->order);
        $address->country = $this->getShippingCountry($this->order);
        $address->zip = $order->get_shipping_postcode();
        $address->locker = $this->getLocker($order);
        return $address;
    }

    private function getShippingCity(\WC_Order $order)
    {
        return !empty($order->get_shipping_city()) ? $order->get_shipping_city() : $order->get_billing_city();
    }

    private function getShippingCountry(\WC_Order $order)
    {
        $countries = \WC()->countries->countries;
        return !empty($countries[$order->get_shipping_country()]) ? $countries[$order->get_shipping_country()] : $order->get_shipping_country();
    }

    private function getBillingCountry(\WC_Order $order)
    {
        $countries = \WC()->countries->countries;
        return !empty($countries[$order->get_billing_country()]) ? $countries[$order->get_billing_country()] : $order->get_billing_country();
    }

    private function getShippingCounty(\WC_Order $order)
    {
        $counties = \WC()->countries->get_states($order->get_shipping_country());
        $county  = !empty($counties[$order->get_shipping_state()]) ? $counties[$order->get_shipping_state()] : $order->get_shipping_state();
        return !empty($county) ? $county : $order->get_shipping_city();
    }

    private function getBillingCounty(\WC_Order $order)
    {
        $counties = \WC()->countries->get_states($order->get_billing_country());
        $county  = !empty($counties[$order->get_billing_state()]) ? $counties[$order->get_billing_state()] : $order->get_billing_state();
        return !empty($county) ? $county : $order->get_billing_city();
    }

    private function getBillingCustomer(\WC_Order $order, $useShippingPhone = false): OrderCustomer
    {
        $customer = new OrderCustomer();
        $customer->first_name = $this->getBillingFirstName($order);
        $customer->last_name = $this->getBillingLastName($order);
        $customer->email = !empty($order->get_billing_email()) ? $order->get_billing_email() : $order->get_order_number() . '@frisbo.eu';
        $customer->phone = $useShippingPhone ? $this->getShippingPhone($order) : $this->getBillingPhone($order);
        $customer->vat_registration_number = $this->getVatRegNumber($order);
        return $customer;
    }

    private function getShippingCustomer(\WC_Order $order): OrderCustomer
    {
        $useBillingForShipping = $this->frisbo->settings->useBillingForShipping ? explode(',', $this->frisbo->settings->useBillingForShipping) : [];
        if (in_array($this->order->get_payment_method(), $useBillingForShipping)) {
            return $this->getBillingCustomer($this->order, true);
        }

        $customer = new OrderCustomer();
        $customer->first_name = $this->getShippingFirstName($order);
        $customer->last_name = $this->getShippingLastName($order);
        $customer->email = !empty($order->get_billing_email()) ? $order->get_billing_email() : $order->get_order_number() . '@frisbo.eu'; //there is no shipping email
        $customer->phone = $this->getShippingPhone($order);
        return $customer;
    }

    private function getBillingPhone(\WC_Order $order)
    {
        $phone = $order->get_billing_phone();
        if (!empty($phone)) {
            return $phone;
        }
        return 0;
    }

    private function getShippingPhone(\WC_Order $order)
    {
        $phone = $order->get_shipping_phone();
        if (!empty($phone)) {
            return $phone;
        }
        return $this->getBillingPhone($order);
    }

    private function getBillingLastName(\WC_Order $order): string
    {
        $company = $order->get_billing_company();
        if ($company) {
            return '.';
        }
        return $order->get_billing_last_name();
    }

    private function getBillingFirstName(\WC_Order $order): string
    {
        $company = $order->get_billing_company();
        if ($company) {
            return $company;
        }
        return $order->get_billing_first_name();
    }

    private function getShippingLastName(\WC_Order $order): string
    {
        $company = $order->get_shipping_company();
        if ($company) {
            return '.';
        }
        return $order->get_shipping_last_name();
    }

    private function getShippingFirstName(\WC_Order $order): string
    {
        $company = $order->get_shipping_company();
        if ($company) {
            return $company;
        }
        return $order->get_shipping_first_name();
    }

    private function getOrderReference(\WC_Order $order): string
    {
        $referenceMeta = $order->get_meta('order_reference');
        if ($referenceMeta) {
            return $referenceMeta;
        }
        return Frisbo::SELLING_CHANNEL_PREFIX . $this->frisbo->settings->orderPrefix . '_' . $order->get_order_number();
    }

    private function getPackingNotes(\WC_Order $order): ?string
    {
        $packingNoteFields = $this->frisbo->settings->packingNoteFields ? explode(',', $this->frisbo->settings->packingNoteFields) : [];
        foreach ($packingNoteFields as $field) {
            $packingNotes = $order->get_meta(trim($field));
            if (!empty($packingNotes)) {
                return $packingNotes;
            }
        }
        return null;
    }

    private function getOrderProducts(\WC_Order $order): array
    {
        $products = [];
        foreach (\apply_filters('frisbo_process_order_get_items', $order->get_items(), $order) as $item_key => $item_values) {
            $products[] = $this->getOrderProduct($item_values);
        }

        foreach ($order->get_items('fee') as $fee) {
            $products[] = $this->getFeeProduct($fee);
        }

        $shippingPrice = $order->get_shipping_total() + $order->get_shipping_tax();
        if ($shippingPrice != 0) {
            $products[] = $this->getShippingProduct($order, $shippingPrice);
        }
        return $products;
    }

    private function getOrderProduct(\WC_Order_Item_Product $itemValues): OrderProduct
    {
        $product = $itemValues->get_product();
        $frisboProduct = new OrderProduct();
        $frisboProduct->name = $itemValues->get_name();
        $frisboProduct->vat = $itemValues->get_subtotal_tax() ? round(($itemValues->get_subtotal_tax()  * 100) / $itemValues->get_subtotal()) : 0;
        $frisboProduct->price_with_vat = ($itemValues->get_subtotal() + $itemValues->get_subtotal_tax()) / $itemValues->get_quantity();
        $frisboProduct->quantity = $itemValues->get_quantity();
        $frisboProduct->discount = 0;
        $frisboProduct->sku = $product->get_sku();
        if ($product->is_virtual() || !in_array($product->get_type(), FrisboProducts::SYNCABLE_PRODUCT_TYPE)) {
            $frisboProduct->sku = null;
            $frisboProduct->is_virtual = true;
        }
        return $frisboProduct;
    }

    private function getShippingProduct(\WC_Order $order, float $shippingPrice): OrderProduct
    {
        $frisboProduct = new OrderProduct();
        $frisboProduct->name = \__('Transport tax', 'frisbo');
        $frisboProduct->vat = $order->get_shipping_tax() ? round(($order->get_shipping_tax()  * 100) / $order->get_shipping_total()) : 0;
        $frisboProduct->price_with_vat = $shippingPrice;
        $frisboProduct->quantity = 1;
        $frisboProduct->is_virtual = true;
        return $frisboProduct;
    }

    private function getFeeProduct(\WC_Order_Item_Fee $fee): OrderProduct
    {
        $frisboProduct = new OrderProduct();
        $frisboProduct->name = $fee->get_name();
        $frisboProduct->vat = (float) $fee->get_total_tax() ? round(((float) $fee->get_total_tax()  * 100) / (float) $fee->get_total()) : 0;
        $frisboProduct->price_with_vat = (float) $fee->get_total_tax() + (float) $fee->get_total();
        $frisboProduct->quantity = 1;
        $frisboProduct->is_virtual = true;
        return $frisboProduct;
    }

    private function getOrderMetadata(\WC_Order $order): array
    {
        $metadata = [];
        $currency = $this->getCurrency($order);
        if ($currency) {
            $metadata['payment_currency'] = $currency;
        }

        $courier = $this->getCourier($order);
        if ($courier) {
            $metadata['courier'] = [
                'courier_name' => $courier
            ];
        }

        return $metadata;
    }

    private function getCurrency(\WC_Order $order): string
    {
        $currencySettings = $this->frisbo->settings->currencyFields ? explode(',', $this->frisbo->settings->currencyFields) : [];
        $currencyFields = array_merge(['payment_currency'], $currencySettings);
        foreach ($currencyFields as $field) {
            $currency = $order->get_meta(trim($field));
            if (!empty($currency)) {
                return $currency;
            }
        }
        return $order->get_currency();
    }

    private function getCourier(\WC_Order $order): ?string
    {
        $courierNameSettings = $this->frisbo->settings->shippingNameMapping ? explode(',', $this->frisbo->settings->shippingNameMapping) : [];
        $shippingMethod = $order->get_items('shipping');
        foreach ($shippingMethod as $item) {
            $itemName = $item->get_name();
            foreach ($courierNameSettings as $setting) {
                $setting = explode(':', trim($setting));
                $shippingMethodName = trim($setting[0]);
                $courierName = trim($setting[1]);
                if ($shippingMethodName == $itemName) {
                    return $courierName;
                }
            }
            return $item->get_name();
        }
        return null;
    }

    private function getVatRegNumber(\WC_Order $order): ?string
    {
        $vatSettings = $this->frisbo->settings->vatRegNumberFields ? explode(',', $this->frisbo->settings->vatRegNumberFields) : [];
        $registrationNumberKeys = ['cuicnp', '_billing_cod_fiscal_cui', 'trade_registration_number'];
        $vatRegNumberFields = array_merge($registrationNumberKeys, $vatSettings);
        foreach ($vatRegNumberFields as $field) {
            $vatRegNumber = $order->get_meta(trim($field));
            if (!empty($vatRegNumber)) {
                return $vatRegNumber;
            }
        }
        return null;
    }

    private function getLocker(\WC_Order $order): ?string
    {
        if (!empty($order->get_meta('_furgonetkaService')) && !empty($order->get_meta('_furgonetkaPoint'))) {
            return $order->get_meta('_furgonetkaService') . '@' . $order->get_meta('_furgonetkaPoint');
        }
        if (!empty($order->get_meta('_packeta_id'))) {
            return 'Packeta@' . $order->get_meta('_packeta_id');
        }

        $lockerFields = $this->frisbo->settings->lockerFields ? explode(',', $this->frisbo->settings->lockerFields) : [];
        foreach ($lockerFields as $field) {
            list($company, $lockerIdField) = explode('@', trim($field));
            $lockerId = $order->get_meta($lockerIdField);

            if ($this->is_valid_json($lockerId)) {
                $lockerId = json_decode($lockerId)->id;
            }

            if (!empty($lockerId)) {
                return "$company@$lockerId";
            }
        }
        return null;
    }

    private function is_valid_json($data): bool
    {
        if (!empty($data)) {
            return is_string($data)
                && is_array(json_decode($data, true)) ? true : false;
        }
        return false;
    }

    /**
     * Get standard woocommerce address if it has values and custom address values based of custom fields inside frisbo settings
     * 
     * @param \WC_Order $order
     * @param bool $isShipping
     * 
     * @return array
     */
    private function getStreet(\WC_Order $order, bool $isShipping = true): array
    {
        $standardAddress = $this->getStandardAddress($order, $isShipping);
        $fields = $this->getAddressSettingFields($isShipping);
        foreach ($fields as $field) {
            $value = $order->get_meta($field);
            if (empty($value)) {
                continue;
            }

            $standardAddress[] = trim($value);
        }

        // remove elements from array where their values are null/empty
        return array_filter($standardAddress, function ($value) {
            return $value !== null && $value !== '';
        });
    }

    /**
     * Get custom address fields from frisbo settings
     * 
     * @param bool $isShipping
     * 
     * @return array
     */
    private function getAddressSettingFields(bool $isShipping): array
    {
        if (!$isShipping) {
            return $this->frisbo->settings->addressBillingFields ? explode(',', $this->frisbo->settings->addressBillingFields) : [];
        }

        return $this->frisbo->settings->addressShippingFields ? explode(',', $this->frisbo->settings->addressShippingFields) : [];
    }

    /**
     * Get standard woocommerce address 
     * 
     * @param \WC_Order $order
     * @param bool $isShipping
     * 
     * @return array
     */
    private function getStandardAddress(\WC_Order $order, bool $isShipping): array
    {
        if (!$isShipping) {
            return [
                $order->get_billing_address_1(),
                $order->get_billing_address_2()
            ];
        }

        return [
            $order->get_shipping_address_1(),
            $order->get_shipping_address_2()
        ];
    }
}
