<?php

namespace Frisbo\WooCommerce;

use Exception;
use Frisbo\FrisboSdk\Exceptions\OrderExists;
use Frisbo\FrisboSdk\Models\Filters\Filter;
use Frisbo\FrisboSdk\Models\Order;

class FrisboOrders
{
    use FrisboTrait;

    private $frisbo;

    const ORDER_CUSTOM_FIELDS = [
        'frisbo_order_status'   => [
            'label'                 => 'Fulfillment status',
            'description'           => 'The status of the order in Frisbo',
        ],
        'reason_status'             => [
            'label'                 => 'Error message',
            'description'           => 'Why your order is not synchronized',
        ],
        'frisbo_order_number'         => [
            'label'                 => 'Order ID',
            'description'           => 'Internal Frisbo order ID',
        ],
        'order_reference'         => [
            'label'                 => 'Order reference',
            'description'           => 'An order reference number',
        ],
        'frisbo_awb_number'         => [
            'label'                 => 'AWB',
            'description'           => 'Order shipping tracking number',
        ],
        'frisbo_awb_tracking_url'         => [
            'label'                 => 'AWB Tracking URL',
            'description'           => 'External courier URL to track AWB',
        ],
        'frisbo_shipped_with'         => [
            'label'                 => 'Courier name',
            'description'           => 'The parcel is being shipped with this courier',
        ],
        'frisbo_awb_label_url'         => [
            'label'                 => 'AWB Label URL',
            'description'           => 'Courier AWB label',
        ],
        'frisbo_invoice_url'         => [
            'label'                 => 'Invoice URL',
            'description'           => 'Invoice document URL',
        ]
    ];

    const ORDER_STATUS = [
        'not-synchronized' => [
            'status' => 'not-synchronized',
            'icon-class' => 'no-alt frisbo-error',
            'title' => 'Not synchronized'
        ],
        'error' => [
            'status' => 'error',
            'icon-class' => 'warning frisbo-error',
            'title' => 'Sync error'
        ],
        'synchronized' => [
            'status' => 'synchronized',
            'icon-class' => 'saved frisbo-success',
            'title' => 'Synchronized'
        ],
        'shipped' => [
            'status' => 'shipped',
            'icon-class' => 'airplane frisbo-success',
            'title' => 'Shipped'
        ],
        'canceled' => [
            'status' => 'canceled',
            'icon-class' => 'trash frisbo-error',
            'title' => 'Canceled'
        ],
        'returned' => [
            'status' => 'returned',
            'icon-class' => 'undo frisbo-warning',
            'title' => 'Returned'
        ],
    ];

    public function __construct()
    {
        $this->frisbo = Frisbo::instance();
        $this->initHooks();
    }

    /**
     * Hook into actions and filters.
     *
     */
    private function initHooks()
    {
        \add_action('woocommerce_admin_order_data_after_order_details', [$this, 'showCustomFields']);
        \add_action('save_post', [$this, 'saveCustomFields']);
        \add_action('woocommerce_thankyou', [$this, 'frisbo_sync_order_auto'], 1, 1);
        \add_action('woocommerce_order_status_cancelled', [$this, 'cancelFrisboOrder']);

        \add_action('wp_ajax_frisbo_sync_orders', [$this, 'sync_orders']);
        \add_action('wp_ajax_nopriv_frisbo_sync_orders', [$this, 'sync_orders']);
        \add_action('wp_ajax_frisbo_sync_status', [$this, 'sync_orders_status']);
        \add_action('wp_ajax_nopriv_frisbo_sync_status', [$this, 'sync_orders_status']);


        \add_action('manage_posts_extra_tablenav', [$this, 'frisbo_sync_all_orders_admin_html']);
        \add_filter('bulk_actions-edit-shop_order', [$this, 'frisbo_register_bulk_order_sync_action']);
        \add_filter('handle_bulk_actions-edit-shop_order', [$this, 'frisbo_sync_order_action_handler'], 10, 3);
        \add_filter('handle_bulk_actions-edit-shop_order',  [$this, 'frisbo_sync_order_status_action_handler'], 10, 3);


        \add_filter('manage_edit-shop_order_columns', [$this, 'showFrisboColumnHead'], 15);
        \add_action('manage_shop_order_posts_custom_column',  [$this, 'showFrisboColumn'], 20, 2);

        \add_filter('woocommerce_admin_order_actions', [$this, 'frisbo_add_custom_order_sync_actions_button'], 100, 2);
        \add_action("wp_ajax_frisbo_sync_order_manually", [$this, "frisbo_sync_order_manually"]);
        \add_action("wp_ajax_nopriv_frisbo_sync_order_manually", [$this, "frisbo_sync_order_manually"]);

        \add_filter('frisbo_process_order_get_items', [$this, 'frisbo_default_product_filter'], 10, 2);
        \add_filter('woocommerce_order_data_store_cpt_get_orders_query', [$this, 'filterOrders'], 10, 2);

        // Cron actions
        \add_action('frisbo_sync_orders_cron', [$this, 'sync_orders_cron']);
        \add_action('frisbo_sync_orders_status_cron', [$this, 'sync_orders_status_cron']);
    }

    public function showCustomFields()
    {
?>
        <div class="frisbo-fields-wrapper">
            <h3 div class="frisbo-fields-title frisbo-success"><?= \__('Frisbo order data', 'frisbo') ?> <span class="dashicons dashicons-arrow-right-alt2"></span></h3>
            <div class="frisbo-fields">
                <?php
                foreach (self::ORDER_CUSTOM_FIELDS as $customField => $args) {
                    $field = [
                        'id' => $customField,
                        'label' => \__($args['label'], 'frisbo'),
                        'class' => 'frisbo-custom-field',
                        'desc_tip' => true,
                        'description' => \__($args['description'], 'frisbo')
                    ];
                    if (!empty($args['custom_attributes']))
                        $field['custom_attributes'] = $args['custom_attributes'];
                    \woocommerce_wp_text_input($field);
                }
                ?>
            </div>
        </div>
        <?php
    }

    public function saveCustomFields($orderId)
    {
        $order = \wc_get_order($orderId);
        if (empty($order))
            return;
        foreach (self::ORDER_CUSTOM_FIELDS as $customField => $args)
            if (isset($_POST[$customField])) {
                $order->update_meta_data($customField, $_POST[$customField]);
                $order->save_meta_data();
            }
    }

    public function sync_orders($timestamp = null, $frisbo = null)
    {
        if (!$frisbo || !($frisbo instanceof Frisbo)) {
            $frisbo = Frisbo::instance();
        }

        if (!$timestamp) {
            $timestamp = time() - \MONTH_IN_SECONDS;
        }

        $status = array_unique($this->getOrderSyncStatus());
        $args = [
            'status' => $status,
            'orderby' => 'id',
            'order' => 'asc',
            'date_created' => '>' . $timestamp,
            'limit' => $frisbo->settings->ordersAmount,
            'frisboMethod' => 'sync_orders'
        ];

        $orders = \wc_get_orders($args);

        foreach ($orders as $order) {
            $this->syncOrder($order, $frisbo);
        }
    }

    public function filterOrders($query, $query_vars)
    {
        if (!empty($query_vars['frisboMethod']) && $query_vars['frisboMethod'] == 'sync_orders') {
            $query['meta_query'][] = array(
                'relation'      => 'OR',
                [
                    'key' => 'frisbo_order_number',
                    'compare' => 'NOT EXISTS',
                ],
                [
                    'key' => 'frisbo_order_number',
                    'value' => ''
                ]
            );
        }

        if (!empty($query_vars['frisboMethod']) && $query_vars['frisboMethod'] == 'sync_order_status') {
            $query['meta_query'][] = array(
                [
                    'key' => 'frisbo_order_number',
                    'compare' => 'EXISTS',
                ],
                [
                    'key' => 'frisbo_order_number',
                    'compare' => '!=',
                    'value' => ''
                ]
            );
            $query['meta_query'][] = array(
                'relation'      => 'OR',
                [
                    'key' => 'frisbo_awb_number',
                    'compare' => 'NOT EXISTS',
                ],
                [
                    'key' => 'frisbo_awb_number',
                    'value' => ''
                ]
            );
            $query['meta_query'][] = array(
                [
                    'key' => 'frisbo_order_status',
                    'compare' => '!=',
                    'value' => self::ORDER_STATUS['canceled']['title'],
                ],
                [
                    'key' => 'frisbo_order_status',
                    'compare' => '!=',
                    'value' => self::ORDER_STATUS['returned']['title'],
                ],
            );
        }

        return $query;
    }

    public function sync_orders_cron()
    {
        if ($this->frisbo->settings->autoFulfillment !== 'on') {
            return;
        }

        $timestamp = time() - (2 * \DAY_IN_SECONDS);
        $this->sync_orders($timestamp, $this->frisbo);
    }

    private function syncOrder(\WC_Order $order, $frisbo = null)
    {
        if (!$frisbo || !($frisbo instanceof Frisbo)) {
            $frisbo = Frisbo::instance();
        }

        if ($this->orderHasErrors($order)) {
            return;
        }

        $responseError = null;
        $orderMappings = (new FrisboOrderDto($order))->getMappings();

        try {
            $response = $frisbo->client->sendOrder($frisbo->settings->organizationId, $orderMappings);
        } catch (OrderExists $e) {
            $frisboData = $this->getOrderInfoFromErrorMessage($frisbo, $e->getMessage());
            $this->updateOrderMeta($order, '', true, $frisboData['order_reference'], $frisboData['order_id']);
            return;
        } catch (Exception $e) {
            $responseError = $e->getMessage();
        }

        if (strpos($responseError, 'field is required') !== false) {
            $errorMessage = '';
            foreach (json_decode($responseError) as $err) {
                $errorMessage .= "{$err[0]}<br/><br/>";
            }
            $this->updateOrderMeta($order, $errorMessage, false);
            return;
        }

        if ($responseError || empty($response->order_id)) {
            $this->updateOrderMeta($order, $responseError, false);
            return;
        }

        $this->updateOrderMeta($order, '', true,  $response->order_reference, $response->order_id);

        if ($frisbo->settings->overwriteStock == 'on') {
            $filter = Filter::from('sku', implode(',', array_column((array) $orderMappings->products, 'sku')), 'in');
            FrisboProducts::syncOrderStock($filter);
        }
    }

    private function orderHasErrors(\WC_Order $order): bool
    {
        $woocomerceOrderStatus = $order->get_status();
        $frisboOrderId = $order->get_meta('frisbo_order_number');
        if ($frisboOrderId || in_array($woocomerceOrderStatus, ['failed', 'cancelled'])) {
            return true;
        }

        if (empty($order->get_items())) {
            $this->updateOrderMeta($order, 'There are no products in this order.', false);
            return true;
        }

        if ($this->orderHasOnlyVirtualProducts($order)) {
            $this->updateOrderMeta($order, 'All products on order are virtual.', false);
            return true;
        }

        $orderSyncStatusSettings = $this->getOrderSyncStatus();
        $paymentMethod =  $order->get_payment_method();
        if (!$this->isStatusPaymentCompatible($order, $orderSyncStatusSettings)) {
            $this->updateOrderMeta($order, "Order with payment method $paymentMethod has to be in status {$orderSyncStatusSettings[$paymentMethod]} to be syncronized.", false);
            return true;
        }

        if (!$this->isStatusCompatible($order, $orderSyncStatusSettings)) {
            $orderStatuses = $this->getOrderSyncStatusWithoutPayment($orderSyncStatusSettings);
            $this->updateOrderMeta($order, "To be syncronized order has to be in status " . implode(', ', $orderStatuses), false);
            return true;
        }

        return false;
    }

    private function isStatusPaymentCompatible($order, $orderSyncStatusSettings)
    {
        $status = $order->get_status();
        $paymentMethod =  $order->get_payment_method();
        if (empty($orderSyncStatusSettings[$paymentMethod])) {
            return true;
        }
        return !empty($orderSyncStatusSettings[$paymentMethod]) && $status == $orderSyncStatusSettings[$paymentMethod];
    }

    private function isStatusCompatible($order, $orderSyncStatusSettings)
    {
        $status = $order->get_status();
        $paymentMethod =  $order->get_payment_method();
        $orderStatus = $this->getOrderSyncStatusWithoutPayment($orderSyncStatusSettings);
        return in_array($status, $orderStatus) || !empty($orderSyncStatusSettings[$paymentMethod]);
    }

    private function getOrderSyncStatusWithoutPayment(array $orderSyncStatus): array
    {
        $orderStatus = [];
        foreach ($orderSyncStatus as $paymentMethod => $status) {
            if (is_numeric($paymentMethod)) {
                $orderStatus[] = $status;
            }
        }
        return $orderStatus;
    }

    private function getOrderSyncStatus(): array
    {
        $orderStatusSettings = $this->frisbo->settings->orderStatusSync ? explode(',', $this->frisbo->settings->orderStatusSync) : [];
        $defaultStatuses = ['processing'];
        if (!$orderStatusSettings) {
            return $defaultStatuses;
        }
        $statuses = [];
        $hasStatusWithoutPayment = false;
        foreach ($orderStatusSettings as $key => $status) {
            $s = explode(':', $status);
            if (count($s) == 1) {
                $statuses[$key] = trim($s[0]);
                $hasStatusWithoutPayment = true;
                continue;
            }
            list($status, $paymentMethod) = $s;
            $statuses[trim($status)] = trim($paymentMethod);
        }
        if (!$hasStatusWithoutPayment) {
            $statuses = array_merge($statuses, $defaultStatuses);
        }
        return $statuses;
    }

    private function updateOrderMeta(\WC_Order $order, string $message, bool $success, string $orderReference = null, string $orderId = null): void
    {
        if ($success === false) {
            $order->update_meta_data('frisbo_order_status', \__(self::ORDER_STATUS['not-synchronized']['title'], 'frisbo'));
            $order->update_meta_data('reason_status', __($message, 'frisbo'));
        }
        if ($success === true) {
            $order->update_meta_data('frisbo_order_status', \__(self::ORDER_STATUS['synchronized']['title'], 'frisbo'));
            $order->update_meta_data('reason_status', '');
        }
        if ($orderReference) {
            $order->update_meta_data('order_reference', $orderReference);
        }
        if ($orderId) {
            $order->update_meta_data('frisbo_order_number', $orderId);
        }
        $order->save_meta_data();
    }

    private function orderHasOnlyVirtualProducts($order)
    {
        foreach ($order->get_items() as $itemValues) {
            if (!$itemValues->get_product()->is_virtual()) {
                return false;
            }
        }
        return true;
    }

    public function frisbo_sync_order_auto($order_id)
    {
        if ($this->frisbo->settings->autoFulfillment != 'on') {
            return;
        }
        $order = new \WC_Order($order_id);
        $this->syncOrder($order, $this->frisbo);
    }

    public function cancelFrisboOrder($orderId)
    {
        $order = \wc_get_order($orderId);
        $frisboOrderId = $order->get_meta('frisbo_order_number');

        if (empty($frisboOrderId))
            return;

        $orderId = new Order();
        $orderId->order_id = $frisboOrderId;

        try {
            $filterByOrderId = new Filter('order_id', $frisboOrderId);
            $frisboOrders = $this->frisbo->client->getOrdersByFilter($this->frisbo->settings->organizationId, $filterByOrderId);
            if (!empty($frisboOrders) && $frisboOrders[0]->status != 'canceled') {
                $this->frisbo->client->transitionOrder($this->frisbo->settings->organizationId, $orderId, 'cancel');
            }
            $order->update_meta_data('frisbo_order_status', self::ORDER_STATUS['canceled']['title']);
            $order->save_meta_data();
        } catch (Exception $e) {
            $order->update_meta_data('reason_status', $e->getMessage());
            $order->save_meta_data();
            return;
        }
    }

    public function sync_orders_status($orders = null, $frisbo = null)
    {
        if (!$frisbo || !($frisbo instanceof Frisbo)) {
            $frisbo = Frisbo::instance();
        }

        if (empty($orders)) {
            $args = array(
                'orderby' => 'modified',
                'order' => 'asc',
                'frisboMethod' => 'sync_order_status',
                'posts_per_page'  => $frisbo->settings->ordersAmount,
            );
            $orders = \wc_get_orders($args);
        }

        $frisboOrderIds = [];
        foreach ($orders as $order) {
            $frisboOrderIds[] = $order->get_meta('frisbo_order_number');
        }

        $this->sync_order_status($frisboOrderIds, $frisbo);

        $this->syncCanceledReturned($frisbo);
    }

    private function syncCanceledReturned(Frisbo $frisbo)
    {
        $filterStatus = Filter::from('status', implode(',', ['canceled', 'returned', 'partially_returned']), 'in');
        $filterDate = Filter::from('created_at', date('Y-m-dTH:i:s', strtotime("-30 days")), 'greater_than');
        $filterChannel = Filter::from('channel_id', $frisbo->settings->channelId);

        $frisboOrders = $frisbo->client->getOrdersByFilter($frisbo->settings->organizationId, $filterStatus, $filterDate, $filterChannel);
        $this->updateOrderStatus($frisboOrders, $frisbo);
    }

    public function sync_orders_status_cron()
    {
        $timestamp = time() - (2 * \MONTH_IN_SECONDS);
        $args = array(
            'orderby' => 'modified',
            'order' => 'asc',
            'date_created' => '>' . $timestamp,
            'frisboMethod' => 'sync_order_status',
            'posts_per_page' => $this->frisbo->settings->ordersAmount,
        );
        $orders = \wc_get_orders($args);
        $this->sync_orders_status($orders, $this->frisbo);
    }

    public function sync_order_status(array $frisboOrderIds, Frisbo $frisbo)
    {
        $orders_per_page = 20;
        $frisboOrderIdsChunks = array_chunk($frisboOrderIds, $orders_per_page);
        foreach ($frisboOrderIdsChunks as $frisboOrderIdsChunk) {
            $filterByOrderId = new Filter('order_id', implode(',', $frisboOrderIdsChunk), 'in');
            $frisboOrders = $frisbo->client->getOrdersByFilter($frisbo->settings->organizationId, $filterByOrderId);
            $this->updateOrderStatus($frisboOrders, $frisbo);
        }
    }

    private function updateOrderStatus(array $frisboOrders, Frisbo $frisbo)
    {
        if (empty($frisboOrders)) {
            return;
        }

        foreach ($frisboOrders as $frisboOrder) {
            $order = \wc_get_order(str_replace(Frisbo::SELLING_CHANNEL_PREFIX . $frisbo->settings->orderPrefix . '_', '', $frisboOrder->order_reference));
            if (!$order) {
                $orderId = $this->getWcOrderIdByFrisboOrderNumber($frisboOrder->order_id);
                $order = \wc_get_order($orderId);
            }
            if (!$order) {
                return;
            }
            if (in_array($frisboOrder->status, ['shipped', 'delivered'])) {
                if (!empty($frisboOrder->shipping_tracking_number) && !empty($frisboOrder->tracking_url)) {
                    $order->update_meta_data('frisbo_awb_number', $frisboOrder->shipping_tracking_number);
                    $order->update_meta_data('frisbo_awb_tracking_url', $frisboOrder->tracking_url);
                    $order->update_meta_data('frisbo_order_status', \__(self::ORDER_STATUS['synchronized']['title'], 'frisbo'));
                    $order->update_meta_data('reason_status', '');
                }
                if (!empty($frisboOrder->shipped_with)) {
                    $order->update_meta_data('frisbo_shipped_with', $frisboOrder->shipped_with);
                }
                $order->update_meta_data('frisbo_awb_label_url', $this->getAttachmentUrl($frisboOrder->organization_id, $frisboOrder->order_id));
                $order->update_meta_data('frisbo_invoice_url', $this->getAttachmentUrl($frisboOrder->organization_id, $frisboOrder->order_id, 'invoice'));
                if (!$order->has_status($frisbo->settings->moveOrderToStatus)) {
                    $order->update_status($frisbo->settings->moveOrderToStatus);
                }
            }
            if (in_array($frisboOrder->status, ['canceled'])) {
                $order->update_meta_data('frisbo_order_status', self::ORDER_STATUS['canceled']['title']);
                if (!$order->has_status($frisbo->settings->moveCanceledOrderToStatus) && $frisbo->settings->moveCanceledOrderToStatus != '-') {
                    $order->update_status($frisbo->settings->moveCanceledOrderToStatus);
                }
            }
            if (in_array($frisboOrder->status, ['returned', 'partially_returned'])) {
                $order->update_meta_data('frisbo_order_status', self::ORDER_STATUS['returned']['title']);
                if (!$order->has_status($frisbo->settings->moveReturnedOrderToStatus) && $frisbo->settings->moveReturnedOrderToStatus != '-') {
                    $order->update_status($frisbo->settings->moveReturnedOrderToStatus);
                }
            }
            $order->save_meta_data();
            \wp_update_post(['ID' => $order->get_id()]);
        }
    }

    public static function getAttachmentUrl(string $organizationId, string $orderId, string $type = 'awb'): string
    {
        $attachmentUrl = 'https://api.frisbo.ro/v1/documents/order/attachment/';
        $attachmentUrl .= base64_encode("$organizationId/$orderId/$type");
        return $attachmentUrl;
    }

    private function getWcOrderIdByFrisboOrderNumber($frisboOrderNumber)
    {
        global $wpdb;
        $results = $wpdb->get_results("SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_value = {$frisboOrderNumber}", \OBJECT);
        return $results[0]->post_id;
    }

    public function frisbo_sync_all_orders_admin_html()
    {
        global $post_type;
        if (!in_array($post_type, ['shop_order'])) {
            return;
        }
        if (!$this->frisbo->hasLoggedIn()) {
            return;
        }
        echo '<div class="alignleft actions"><input type="submit" name="sync_action" id="syncAllOrders" class="button button-primary" value="' . \__('Sync orders', 'frisbo') . '"></div>';
        echo '<div class="alignleft actions"><input type="submit" name="sync_action" id="syncStatus" class="button button-primary" value="' . \__('Sync status', 'frisbo') . '"></div>';
    }

    public function frisbo_register_bulk_order_sync_action($bulk_actions)
    {
        if (!$this->frisbo->hasLoggedIn()) {
            return $bulk_actions;
        }
        $bulk_actions['sync_orders_frisbo'] = \__('Sync orders with Frisbo', 'frisbo');
        $bulk_actions['sync_orders_status_frisbo'] = \__('Sync orders status with Frisbo', 'frisbo');
        return $bulk_actions;
    }


    public function frisbo_sync_order_action_handler($redirect_to, $doaction, $post_ids)
    {
        if ($doaction !== 'sync_orders_frisbo') {
            return false;
        }
        foreach ($post_ids as $post_id) {
            $order = \wc_get_order($post_id);
            $this->syncOrder($order, $this->frisbo);
        }

        \wp_redirect(\wp_get_referer());
    }

    public function frisbo_sync_order_status_action_handler($redirect_to, $doaction, $post_ids)
    {
        if ($doaction !== 'sync_orders_status_frisbo') {
            return false;
        }
        $frisboOrderIds = [];
        foreach ($post_ids as $post_id) {
            $frisboOrderIds[] = \get_post_meta($post_id, 'frisbo_order_number', true);
        }
        $this->sync_order_status($frisboOrderIds, $this->frisbo);
        \wp_redirect(\wp_get_referer());
    }

    public function showFrisboColumnHead($columns)
    {
        $newColumns = array();
        foreach ($columns as $key => $column) {
            $newColumns[$key] =  $columns[$key];
            if ($key === 'order_status')
                $newColumns['frisbo_order_status'] = 'Frisbo';
        }
        return $newColumns;
    }

    public function showFrisboColumn($column, $postid)
    {
        if ($column == 'frisbo_order_status') {
            $order = \wc_get_order($postid);
            $status = $this->orderSyncStatus($order);
        ?>
            <div class="wrapper-prevent-default">
                <div class="frisbo-status">
                    <div class="frisbo-status-icon frisbo-status-sync">
                        <span class="dashicons dashicons-<?= self::ORDER_STATUS[$status]['icon-class'] ?>"></span>
                        <div class="frisbo-hover-info">
                            <div class="frisbo-info-title"><?= \__(self::ORDER_STATUS[$status]['title'], 'frisbo') ?></div>
                            <div class="frisbo-info-wrapper">
                                <?php if ($status == self::ORDER_STATUS['error']['status']) : ?>
                                    <div class="frisbo-error"><?= $order->get_meta('reason_status') ?></div>
                                <?php elseif ($status == self::ORDER_STATUS['synchronized']['status']) : ?>
                                    <div class="frisbo-id" title="<?= \__('Frisbo Order ID', 'frisbo') ?>"><?= $order->get_meta('frisbo_order_number') ?></div>
                                <?php elseif ($status == self::ORDER_STATUS['shipped']['status']) : ?>
                                    <div class="frisbo-awb">
                                        <span title="<?= \__('Frisbo Order ID', 'frisbo') ?>"><?= $order->get_meta('frisbo_order_number') ?></span>
                                        &nbsp;<span class="dashicons dashicons-admin-links"></span>&nbsp;
                                        <a title="<?= \__('AWB', 'frisbo') ?>" href="<?= $order->get_meta('frisbo_awb_tracking_url') ?>" target="_blank">
                                            <?= $order->get_meta('frisbo_awb_number') ?>
                                        </a>
                                    </div>
                                <?php endif; ?>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
<?php
        }
    }

    private function orderSyncStatus($order)
    {
        if (!empty($order->get_meta('reason_status'))) {
            return self::ORDER_STATUS['error']['status'];
        }
        if ($order->get_meta('frisbo_order_status') == self::ORDER_STATUS['canceled']['title']) {
            return self::ORDER_STATUS['canceled']['status'];
        }
        if ($order->get_meta('frisbo_order_status') == self::ORDER_STATUS['returned']['title']) {
            return self::ORDER_STATUS['returned']['status'];
        }
        if (!empty($order->get_meta('frisbo_awb_number'))) {
            return self::ORDER_STATUS['shipped']['status'];
        }
        if (empty($order->get_meta('frisbo_order_number'))) {
            return self::ORDER_STATUS['not-synchronized']['status'];
        }
        return self::ORDER_STATUS['synchronized']['status'];
    }

    public function frisbo_add_custom_order_sync_actions_button($actions, $order)
    {
        $frisboOrderId = $order->get_meta('frisbo_order_number');
        if (!$frisboOrderId) {
            // The key slug defined for the action button
            $action_slug = 'frisbo_sync_order_manually';
            $id = $order->get_id();
            // Set the action button
            $actions[$action_slug] = array(
                'url'       => \wp_nonce_url(\admin_url('admin-ajax.php?action=frisbo_sync_order_manually&order_id=' . $id), 'woocommerce-mark-order-status'),
                'name'      => \__('Sync with Frisbo', 'frisbo'),
                'action'    => $action_slug,
            );
        }
        return $actions;
    }

    public function frisbo_sync_order_manually()
    {
        $order = \wc_get_order($_REQUEST['order_id']);
        $this->syncOrder($order);
        \wp_redirect($_SERVER['HTTP_REFERER']);
        exit;
    }

    public function frisbo_default_product_filter($orderItems, $order)
    {
        return $orderItems;
    }
}

new FrisboOrders();
