<?php

namespace Frisbo\WooCommerce;

use Exception;
use Frisbo\FrisboSdk\Exceptions\ProductExists;
use Frisbo\FrisboSdk\Models\Filters\Filter;
use WP_Query;

class FrisboProducts
{
    use FrisboTrait;

    /**
     * Frisbo instance
     *
     * @var Frisbo
     */
    private $frisbo;

    /**
     * The single instance of the class.
     *
     * @var FrisboProducts
     */
    private static $instance = null;

    /**
     * Woocommerce product types that can be syncronized with Frisbo.
     */
    const SYNCABLE_PRODUCT_TYPE = [
        'simple',
        'variation'
    ];

    const PRODUCT_CUSTOM_FIELDS = [
        'frisbo_inventory_status'   => [
            'label'                 => 'Inventory status',
            'description'           => 'The status of the product in Frisbo',
        ],
        'reason_status'             => [
            'label'                 => 'Error message',
            'description'           => 'Why your product is not synchronized',
        ],
        'frisbo_product_id'         => [
            'label'                 => 'Product ID',
            'description'           => 'Internal Frisbo product ID'
        ],
        'stock_last_updated'         => [
            'label'                 => 'Stock last updated',
            'description'           => 'When the stock was last syncronized with Frisbo'
        ]
    ];

    const PRODUCT_STATUS = [
        'not-synchronized' => [
            'status' => 'not-synchronized',
            'icon-class' => 'no-alt frisbo-error',
            'title' => 'Not synchronized'
        ],
        'error' => [
            'status' => 'error',
            'icon-class' => 'warning frisbo-error',
            'title' => 'Sync error'
        ],
        'unsyncable' => [
            'status' => 'unsyncable',
            'icon-class' => 'info frisbo-info',
            'title' => 'Unsyncable product'
        ],
        'partially' => [
            'status' => 'partially',
            'icon-class' => 'warning frisbo-warning',
            'title' => 'Partially synchronized'
        ],
        'synchronized' => [
            'status' => 'synchronized',
            'icon-class' => 'saved frisbo-success',
            'title' => 'Synchronized'
        ],
    ];

    /**
     * Ensures only one instance of Frisbo Products is loaded or can be loaded.
     *
     * @return FrisboProducts - Main instance.
     */
    public static function instance(): self
    {
        if (null === self::$instance) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function __construct()
    {
        $this->frisbo = Frisbo::instance();
        $this->initHooks();
    }

    /**
     * Hook into actions and filters.
     *
     */
    private function initHooks()
    {
        \add_action('woocommerce_product_options_general_product_data', [$this, 'showCustomFields']);
        \add_action('woocommerce_admin_process_product_object', [$this, 'saveCustomFields']);

        \add_action('woocommerce_product_after_variable_attributes', [$this, 'showVariationCustomFields'], 10, 3);
        \add_action('woocommerce_save_product_variation', [$this, 'saveVariationCustomFields']);

        \add_filter('woocommerce_duplicate_product_exclude_meta', [$this, 'excludeDuplication'], 10, 2);

        \add_filter('bulk_actions-edit-product', [$this, 'bulkActions']);
        \add_filter('handle_bulk_actions-edit-product', [$this, 'bulkSyncAction'], 10, 3);
        \add_filter('handle_bulk_actions-edit-product', [$this, 'bulkStockAction'], 10, 3);
        \add_filter('woocommerce_product_data_store_cpt_get_products_query', [$this, 'filterProducts'], 10, 2);

        \add_filter('manage_edit-product_columns',  [$this, 'showFrisboColumnHead'], 15);
        \add_action('manage_product_posts_custom_column',  [$this, 'showFrisboColumn'], 10, 2);

        \add_action('manage_posts_extra_tablenav',  [$this, 'addSyncButtons']);

        \add_action('wp_ajax_frisbo_sync_products', [$this, 'sync_products']);
        \add_action('wp_ajax_nopriv_frisbo_sync_products', [$this, 'sync_products']);
        \add_action('wp_ajax_frisbo_sync_stock',  [$this, 'sync_stock']);
        \add_action('wp_ajax_nopriv_frisbo_sync_stock',  [$this, 'sync_stock']);

        // Cron actions
        \add_action('frisbo_sync_products_cron', [$this, 'sync_products_cron']);
        \add_action('frisbo_sync_products_stock_cron', [$this, 'sync_stock_cron']);
        \add_action('frisbo_sync_last_products_stock_cron', [$this, 'sync_stock_last_products']);
    }

    public function showCustomFields($productId)
    {
        $product = \wc_get_product(\get_the_ID());
        if ($product->get_type() == 'variable')
            return;
?>
        <div class="frisbo-fields-wrapper">
            <h3 div class="frisbo-fields-title frisbo-success"><?= \__('Frisbo product data', 'frisbo') ?> <span class="dashicons dashicons-arrow-right-alt2"></span></h3>
            <div class="frisbo-fields">
                <?php
                foreach (self::PRODUCT_CUSTOM_FIELDS as $customField => $args) {
                    $field = [
                        'id' => $customField,
                        'label' => \__($args['label'], 'frisbo'),
                        'class' => 'frisbo-custom-field',
                        'desc_tip' => true,
                        'description' => \__($args['description'], 'frisbo')
                    ];
                    if (!empty($args['custom_attributes']))
                        $field['custom_attributes'] = $args['custom_attributes'];
                    \woocommerce_wp_text_input($field);
                }
                ?>
            </div>
        </div>
    <?php
    }

    public function showVariationCustomFields($loop, $variation_data, $variation)
    {
    ?>
        <div class="frisbo-fields-wrapper">
            <h3 div class="frisbo-fields-title frisbo-success"><?= \__('Frisbo product data', 'frisbo') ?> <span class="dashicons dashicons-arrow-right-alt2"></span></h3>
            <div class="frisbo-fields">
                <?php
                foreach (self::PRODUCT_CUSTOM_FIELDS as $customField => $args) {
                    $field = [
                        'id' => $customField . $variation->ID,
                        'label' => \__($args['label'], 'frisbo'),
                        'class' => 'frisbo-custom-field',
                        'desc_tip' => true,
                        'description' => \__($args['description'], 'frisbo'),
                        'value'       => \get_post_meta($variation->ID, $customField, true)
                    ];
                    if (!empty($args['custom_attributes']))
                        $field['custom_attributes'] = $args['custom_attributes'];
                    \woocommerce_wp_text_input($field);
                }
                ?>
            </div>
        </div>
        <?php
    }

    public function saveCustomFields($product)
    {
        foreach (self::PRODUCT_CUSTOM_FIELDS as $customField => $args)
            if (isset($_POST[$customField])) {
                $product->update_meta_data($customField, $_POST[$customField]);
                $product->save_meta_data();
            }
    }

    public function saveVariationCustomFields($variationId)
    {
        foreach (self::PRODUCT_CUSTOM_FIELDS as $customField => $args)
            if (isset($_POST[$customField . $variationId])) {
                \update_post_meta($variationId, $customField, $_POST[$customField . $variationId]);
            }
    }

    public function excludeDuplication()
    {
        return array_keys(self::PRODUCT_CUSTOM_FIELDS);
    }

    public function bulkActions($bulk_actions)
    {
        if (!$this->frisbo->hasLoggedIn()) {
            return $bulk_actions;
        }

        $bulk_actions['sync_products_frisbo'] = \__('Sync products with Frisbo', 'frisbo');
        if ($this->frisbo->settings->overwriteStock == 'on') {
            $bulk_actions['sync_products_stock_frisbo'] = \__('Sync stock with Frisbo', 'frisbo');
        }

        return $bulk_actions;
    }

    public function bulkSyncAction($redirect_to, $doaction, $post_ids)
    {
        if ($doaction !== 'sync_products_frisbo') {
            return false;
        }
        foreach ($post_ids as $post_id) {
            $product = \wc_get_product($post_id);
            if ($product->get_type() == 'variable') {
                $variationsId = $product->get_children();
                foreach ($variationsId as $variationId) {
                    $productVariation = new \WC_Product_Variation($variationId);
                    $this->sync_product($productVariation, $this->frisbo);
                }
                continue;
            }
            $this->sync_product($product, $this->frisbo);
        }

        \wp_redirect(\wp_get_referer());
    }

    public function bulkStockAction($redirect_to, $doaction, $post_ids)
    {
        if ($doaction !== 'sync_products_stock_frisbo') {
            return false;
        }

        $frisboProductIds = [];
        foreach ($post_ids as $post_id) {
            $product = \wc_get_product($post_id);
            if ($product->get_type() == 'variable') {
                $variationsId = $product->get_children();
                foreach ($variationsId as $variationId) {
                    $productVariation = new \WC_Product_Variation($variationId);
                    $frisboProductId = $productVariation->get_meta('frisbo_product_id');
                    if (!empty($frisboProductId))
                        $frisboProductIds[] = $frisboProductId;
                }
                continue;
            }
            $frisboProductId = $product->get_meta('frisbo_product_id');
            if (!empty($frisboProductId))
                $frisboProductIds[] = $frisboProductId;
        }

        $filter = Filter::from('product_id', implode(',', $frisboProductIds), 'in');
        $this->sync_stock($filter, $this->frisbo);

        \wp_redirect(\wp_get_referer());
    }

    public function showFrisboColumnHead($columns)
    {
        $newColumns = array();
        foreach ($columns as $key => $column) {
            $newColumns[$key] =  $columns[$key];
            if ($key === 'name')
                $newColumns['frisbo_status'] = 'Frisbo';
        }
        return $newColumns;
    }

    /**
     * Add frisbo status in columns for every product
     */
    public function showFrisboColumn($column, $postid)
    {
        if ($column == 'frisbo_status') :
            $product = \wc_get_product($postid);
            $productType = $product->get_type();
            $status = $this->syncStatus($product);
        ?>
            <div class="frisbo-status">
                <div class="frisbo-status-icon frisbo-status-sync">
                    <span class="dashicons dashicons-<?= self::PRODUCT_STATUS[$status]['icon-class'] ?>"></span>
                    <div class="frisbo-hover-info">
                        <div class="frisbo-info-title"><?= \__(self::PRODUCT_STATUS[$status]['title'], 'frisbo') ?></div>
                        <div class="frisbo-info-wrapper">
                            <?php switch ($productType):
                                case 'simple': ?>
                                    <?php if ($status == self::PRODUCT_STATUS['error']['status']) : ?>
                                        <div class="frisbo-error"><?= $product->get_meta('reason_status') ?></div>
                                    <?php elseif ($status == self::PRODUCT_STATUS['synchronized']['status']) : ?>
                                        <div class="frisbo-id" title="<?= \__('Frisbo Product ID', 'frisbo') ?>"><?= $product->get_meta('frisbo_product_id') ?></div>
                                    <?php endif; ?>
                                <?php break;
                                case 'variable': ?>
                                    <?php
                                    $variationsId = $product->get_children();
                                    foreach ($variationsId as $variationId) :
                                        $productVariation = new \WC_Product_Variation($variationId);
                                        $variationStatus = $this->syncStatus($productVariation);
                                    ?>
                                        <div class="frisbo-product-variation-status <?= $variationStatus == self::PRODUCT_STATUS['error']['status'] ? 'variation-auto-height' : '' ?>">
                                            <span class="frisbo-variation-id"><?= $variationId ?></span>
                                            <span class="dashicons dashicons-<?= self::PRODUCT_STATUS[$variationStatus]['icon-class'] ?>" title="<?= \__(self::PRODUCT_STATUS[$variationStatus]['title'], 'frisbo') ?>"></span>
                                            <?php
                                            $lastUpdated = $productVariation->get_meta('stock_last_updated');
                                            if (!empty($lastUpdated)) :
                                                $lastUpdateClass = $this->daysPassedLastSync($lastUpdated) >= 1 ? 'frisbo-info' : 'frisbo-success';
                                            ?>
                                                <span class="dashicons dashicons-clock <?= $lastUpdateClass ?>" title="<?= \__('Stock last updated at', 'frisbo') ?> <?= $lastUpdated ?>"></span>
                                            <?php elseif (!empty($productVariation->get_meta('frisbo_product_id'))) : ?>
                                                <span class="dashicons dashicons-marker frisbo-warning" title="<?= \__('Unsynchronized stock', 'frisbo') ?>"></span>
                                            <?php endif; ?>
                                            <?php if ($variationStatus == self::PRODUCT_STATUS['error']['status']) : ?>
                                                <div class="frisbo-message frisbo-error"> <?= $productVariation->get_meta('reason_status') ?></div>
                                            <?php elseif ($variationStatus == self::PRODUCT_STATUS['synchronized']['status']) : ?>
                                                <span class="variation-info" title="<?= \__('Frisbo Product ID', 'frisbo') ?>"><?= $productVariation->get_meta('frisbo_product_id') ?></span>
                                            <?php endif; ?>
                                        </div>
                                    <?php endforeach; ?>
                                <?php break;
                                default: ?>
                                    <div><?= \__("Product type $productType is unsyncable. We'll treat it as a virtual product.", 'frisbo') ?></div>
                            <?php endswitch; ?>
                        </div>
                    </div>
                </div>
                <?php if (in_array($productType, ['simple']) && $status == self::PRODUCT_STATUS['synchronized']['status']) : ?>
                    <div class="frisbo-status-icon frisbo-status-stock">
                        <?php
                        $lastUpdated = $product->get_meta('stock_last_updated');
                        if (!empty($lastUpdated)) :
                            $lastUpdateClass = $this->daysPassedLastSync($lastUpdated) >= 1 ? 'frisbo-info' : 'frisbo-success';
                        ?>
                            <span class="dashicons dashicons-clock <?= $lastUpdateClass ?>"></span>
                            <div class="frisbo-hover-info"><?= \__('Stock last updated at', 'frisbo') ?><br /><?= $lastUpdated ?></div>
                        <?php else : ?>
                            <span class="dashicons dashicons-marker frisbo-warning"></span>
                            <div class="frisbo-hover-info"><?= \__('Unsynchronized stock', 'frisbo') ?></div>
                        <?php endif; ?>
                    </div>
                <?php endif; ?>
            </div>
<?php
        endif;
    }

    private function daysPassedLastSync($date)
    {
        $now = time();
        $your_date = strtotime($date);
        $datediff = $now - $your_date;

        return $datediff / (60 * 60 * 24);
    }

    public function addSyncButtons()
    {
        global $post_type;
        if (!in_array($post_type, ['product'])) {
            return;
        }

        if ($this->frisbo->hasLoggedIn()) {
            echo '<div class="alignleft actions"><input type="submit" name="sync_action" id="syncAllProducts" class="button button-primary" value="' . \__('Sync products', 'frisbo') . '"></div>';
        }

        if ($this->frisbo->hasLoggedIn() && $this->frisbo->settings->overwriteStock == 'on') {
            echo '<div class="alignleft actions"><input type="submit" name="sync_action" id="syncStock" class="button button-primary" value="' . \__('Sync stock', 'frisbo') . '"></div>';
        }
    }

    public function sync_products($frisbo = null)
    {
        if (!$frisbo || !($frisbo instanceof Frisbo)) {
            $frisbo = Frisbo::instance();
        }

        // TODO: to be removed
        $this->removeFrisboIdFromParentVariableProducts();

        $products = \wc_get_products([
            'type'              => self::SYNCABLE_PRODUCT_TYPE,
            'posts_per_page'    => $frisbo->settings->productsAmount,
            'orderby'           => 'modified',
            'order'             => 'asc',
            'frisboMethod'      => 'sync_products'
        ]);

        foreach ($products as $product)
            $this->sync_product($product, $frisbo);
    }

    public function sync_products_cron()
    {
        if ($this->frisbo->settings->autoFulfillment !== 'on') {
            return;
        }

        $this->sync_products($this->frisbo);
    }

    public function filterProducts($query, $query_vars)
    {
        if (!empty($query_vars['frisboMethod']) && $query_vars['frisboMethod'] == 'sync_stock_last_products') {
            $query['meta_query'][] = [
                'relation' => 'AND',
                [
                    'key'          => 'frisbo_product_id',
                    'compare'      => 'EXISTS',
                ],
                [
                    'key'          => 'frisbo_product_id',
                    'value'        => [''],
                    'compare'      => 'NOT IN',
                ],
                [
                    'key'          => '_virtual',
                    'value'        => 'no',
                ],
            ];
        }
        if (!empty($query_vars['frisboMethod']) && $query_vars['frisboMethod'] == 'sync_products') {
            $query['meta_query'][] = [
                'relation' => 'OR',
                [
                    'key'          => 'frisbo_product_id',
                    'compare'      => 'NOT EXISTS',
                ],
                [
                    'key'          => 'frisbo_product_id',
                    'value'        => ''
                ]
            ];
        }

        return $query;
    }


    private function removeFrisboIdFromParentVariableProducts()
    {
        $action = 'remove-frisbo-id-from-variable-products';
        if (!\get_option($action)) {
            \update_option($action, 1);

            $products = \wc_get_products([
                'type' => 'variable',
                'meta_key' =>  'frisbo_product_id',
                'meta_value' => '',
                'meta_compare' => '!='
            ]);
            foreach ($products as $product) {
                $product->update_meta_data('frisbo_product_id', '');
                $product->save_meta_data();
            }
        }
    }

    private function sync_product($product, Frisbo $frisbo)
    {
        $productType = $product->get_type();
        if (!in_array($productType, self::SYNCABLE_PRODUCT_TYPE)) {
            $this->updateProductMeta($product, \__("Product type $productType is unsyncable. We'll treat it as a virtual product.", 'frisbo'), false);
            return;
        }

        $productMappings = (new FrisboProductDto($product))->getMappings();

        $error = null;
        try {
            $frisboProduct = $frisbo->client->sendProduct($frisbo->settings->organizationId, $productMappings);
        } catch (ProductExists $e) {
            $this->updateProductMeta($product, '', true, $this->getProductIdFromErrorMessage($e->getMessage()));
            return;
        } catch (Exception $e) {
            $error = $e->getMessage();
        }

        $this->syncWooProduct($product, $frisboProduct, $error);
        \wp_update_post(['ID' => $product->get_id()]);
    }

    private function syncWooProduct($wooProduct, $frisboProduct = null, $error)
    {
        if (strpos($error, 'sku field is required') !== false) {
            $error = \__('SKU field is not set', 'frisbo');
        }

        if ($error || !$frisboProduct) {
            $this->updateProductMeta($wooProduct, $error, false);
            return;
        }

        $this->updateProductMeta($wooProduct, '', true, $frisboProduct->product_id);
    }

    private function updateProductMeta($product, string $message, bool $success, string $productId = null)
    {
        if ($success === false) {
            $product->update_meta_data('frisbo_inventory_status', \__(self::PRODUCT_STATUS['not-synchronized']['title'], 'frisbo'));
            $product->update_meta_data('reason_status', $message);
        }
        if ($success === true) {
            $product->update_meta_data('frisbo_inventory_status', \__(self::PRODUCT_STATUS['synchronized']['title'], 'frisbo'));
            $product->update_meta_data('reason_status', '');
        }
        if ($productId) {
            $product->update_meta_data('frisbo_product_id', $productId);
        }
        $product->save_meta_data();
    }

    public function sync_stock_last_products()
    {
        $args = [
            'type'              => self::SYNCABLE_PRODUCT_TYPE,
            'orderby'           => 'meta_value',
            'meta_key'          => 'stock_last_updated',
            'limit'             => $this->frisbo->settings->productsAmount,
            'order'             => 'ASC',
            'frisboMethod'      => 'sync_stock_last_products'
        ];

        if (function_exists('pll_default_language')) {
            $args['lang'] = pll_default_language('slug');
        }

        $products = \wc_get_products($args);

        $frisboProductIds = [];
        foreach ($products as $product) {
            $frisboProductId = get_post_meta($product->get_id(), 'frisbo_product_id', true);
            $frisboProductIds[] = $frisboProductId;
        }

        $filter = Filter::from('product_id', implode(',', $frisboProductIds), 'in');
        $this->sync_stock($filter);
    }

    public function sync_stock($filter = null, $frisbo = null)
    {
        if (!$frisbo || !($frisbo instanceof Frisbo)) {
            $frisbo = Frisbo::instance();
        }

        if ($frisbo->settings->overwriteStock !== 'on') {
            return;
        }

        if (!$filter) {
            $filter = Filter::from('updated_at', date('Y-m-dTH:i:s', strtotime("-1 year")), 'greater_than');
        }

        if ($frisbo->settings->noStock == 'on') {
            $noStockFilter = Filter::from('withNoStorage', true);
            $response = $frisbo->client->getStocksByFilter($frisbo->settings->organizationId, $frisbo->settings->channelId, $filter, $noStockFilter);
        } else {
            $response = $frisbo->client->getStocksByFilter($frisbo->settings->organizationId, $frisbo->settings->channelId, $filter);
        }

        $frisboIdMappedStorages = [];
        foreach ($response as $storage) {
            if ($storage) {
                $frisboIdMappedStorages[$storage->product_id] = $storage;
            }
        }

        $this->updateProductsStock($frisboIdMappedStorages, $frisbo);
    }

    public function sync_stock_cron()
    {
        $filter = Filter::from('updated_at', date('Y-m-dTH:i:s', strtotime("-14 days")), 'greater_than');
        $this->sync_stock($filter, $this->frisbo);
    }

    private function updateProductsStock($frisboIdMappedStorages, Frisbo $frisbo)
    {
        $chunks = array_chunk($frisboIdMappedStorages, $frisbo->settings->stockAmount, true);
        foreach ($chunks as $productChunks) {
            $frisboProductIds = array_keys($productChunks);

            $products = \wc_get_products([
                'type'             => self::SYNCABLE_PRODUCT_TYPE,
                'meta_key' =>  'frisbo_product_id',
                'meta_value' => $frisboProductIds,
                'meta_compare' => 'IN',
                // apparently lang from theme can alter queries ... wp sucks
                // https://stackoverflow.com/questions/30213495/wp-query-returns-no-results
                'suppress_filters' => true
            ]);

            foreach ($products as $product) {
                if (!$product->is_virtual()) {
                    $productFrisboId = \get_post_meta($product->get_id(), 'frisbo_product_id', true);
                    $this->updateStock($product, $frisboIdMappedStorages[$productFrisboId]);
                }
            }
        }
    }

    private function updateStock($wooProduct, $frisboProduct)
    {
        if ($wooProduct->get_type() === 'variable') {
            return;
        }

        if (!$wooProduct->get_manage_stock()) {
            $wooProduct->set_manage_stock('true');
        }

        $wooProduct->set_stock_quantity($frisboProduct->available_stock);
        $wooProduct->update_meta_data('stock_last_updated', date('Y-m-d H:i:s'));
        $wooProduct->save_meta_data();
        $wooProduct->save();
    }

    private function syncStatus($product)
    {
        if (!in_array($product->get_type(), ['simple', 'variable', 'variation']))
            return self::PRODUCT_STATUS['unsyncable']['status'];
        if ($product->get_type() == 'variable') {
            $childrensId = $product->get_children();
            $noSyncProducts = 0;
            foreach ($childrensId as $variationId) {
                $productVariation = new \WC_Product_Variation($variationId);
                if (!empty($productVariation->get_meta('frisbo_product_id')))
                    $noSyncProducts++;
            }
            if (count($childrensId) == $noSyncProducts && $noSyncProducts > 0)
                return self::PRODUCT_STATUS['synchronized']['status'];
            if (count($childrensId) !== $noSyncProducts && $noSyncProducts > 0)
                return self::PRODUCT_STATUS['partially']['status'];
            return self::PRODUCT_STATUS['not-synchronized']['status'];
        }
        if (!empty($product->get_meta('reason_status')))
            return self::PRODUCT_STATUS['error']['status'];
        if (empty($product->get_meta('frisbo_product_id')))
            return self::PRODUCT_STATUS['not-synchronized']['status'];
        return self::PRODUCT_STATUS['synchronized']['status'];
    }

    public static function syncOrderStock($filter)
    {
        $productsInstance = FrisboProducts::instance();
        $productsInstance->sync_stock($filter, $productsInstance->frisbo);
    }
}

FrisboProducts::instance();
