<?php
namespace Frisbo\WooCommerce;

use Exception;
use Frisbo\FrisboSdk\Models\Filters\Filter;
use Frisbo\FrisboSdk\Models\Order;
use Frisbo\FrisboSdk\Models\Product;

trait FrisboTrait
{
    private function getProductIdFromErrorMessage(string $errorMessage): int
    {
        $messageFormat = "Product with sku/id %[^/]/%d already exists";
        list($sku, $id) = sscanf($errorMessage, $messageFormat);
        return $id;
    }

    private function getOrderInfoFromErrorMessage(Frisbo $frisbo, string $frisboMessage): array
    {
        $messageFormat = "Order with reference/id " . Frisbo::SELLING_CHANNEL_PREFIX . $frisbo->settings->orderPrefix . "_%d/%s already exists.";
        list($frisboOrderReference, $frisboOrderId) = sscanf($frisboMessage, $messageFormat);

        return [
            'order_reference' => Frisbo::SELLING_CHANNEL_PREFIX . $frisbo->settings->orderPrefix . "_" . $frisboOrderReference,
            'order_id' => $frisboOrderId
        ];
    }

    private function exceptionInfo(Exception $ex): string
    {
        return json_encode([
            'message' => $ex->getMessage(),
            'file' => $ex->getFile(),
            'line' => $ex->getLine(),
            'trace' => $ex->getTraceAsString()
        ]);
    }

    private function waitFrisboProduct($sku): Product
    {
        $frisbo = Frisbo::instance();
        $timeSpent = 1;
        while ($timeSpent < Frisbo::TIMEOUT) {
            sleep(1);
            $frisboProduct = $frisbo->client->getProductsByFilter($frisbo->settings->organizationId, Filter::from('sku', $sku))[0];
            $this->log("Waited $timeSpent for product with sku $sku to be created.");
            if (!empty($frisboProduct)) {
                break;
            }
            $timeSpent += 1;
        }
        return $frisboProduct;
    }

    private function waitFrisboOrder($orderNumber, $status = ['new', 'open', 'invoiced']): Order
    {
        $frisbo = Frisbo::instance();
        $timeSpent = 1;
        while ($timeSpent < Frisbo::TIMEOUT) {
            sleep(1);
            $orderReferenceFilter = Filter::from('order_reference', Frisbo::SELLING_CHANNEL_PREFIX . $frisbo->settings->orderPrefix . '_' . $orderNumber);
            $frisboOrder = $frisbo->client->getOrdersByFilter($frisbo->settings->organizationId, $orderReferenceFilter)[0];
            $this->log("Waited $timeSpent for order with id {$frisboOrder->order_id} to be in status {$frisboOrder->status}");
            if (!empty($frisboOrder) && in_array($frisboOrder->status, $status)) {
                break;
            }
            if ($frisboOrder->fulfillment->status == 'Pending fulfillment') {
                $frisbo->client->transitionOrder($frisbo->settings->organizationId, $frisboOrder, 'confirmFulfillment');
            }
            $timeSpent += 1;
        }
        return $frisboOrder;
    }

    private function getRand($low = false)
    {
        if ($low) {
            return rand(1000, 9999);
        }
        return time() . rand(123456, 999999);
    }

    private function getRandFloat(): float
    {
        return $this->getRand(true) / 100;
    }

    /**
     * Output is sent to the terminal
     *
     * @param string $s
     * @return void
     */
    private function log(string $s)
    {
        fwrite(STDERR, print_r($s . PHP_EOL, TRUE));
    }
}
