<?php

namespace Frisbo\WooCommerce;


use Exception;
use Frisbo\FrisboSdk\Exceptions\AccessForbiden;
use Frisbo\FrisboSdk\Exceptions\OrderExists;
use Frisbo\FrisboSdk\Exceptions\ProductExists;
use Frisbo\FrisboSdk\Exceptions\ResponseErrorHandler;
use Frisbo\FrisboSdk\Exceptions\ServerErrorException;
use Frisbo\FrisboSdk\FrisboV1Client;
use Frisbo\FrisboSdk\Models\Channel;
use Frisbo\FrisboSdk\Models\Filters\Filter;
use Frisbo\FrisboSdk\Models\Order;
use Frisbo\FrisboSdk\Models\Organization;
use Frisbo\FrisboSdk\Models\Product;
use Frisbo\FrisboSdk\Models\Storage;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;

class FrisboWordpressClient extends FrisboV1Client implements LoggerAwareInterface
{
    use LoggerAwareTrait, ResponseErrorHandler;


    const HTTP_ARGS = [
        'timeout' => 30.0,
        'Content-Type' => 'application/json'
    ];

    /**
     * Login via username and password and return json
     *
     * @param string $username
     * @param string $password
     * @return string|null
     */
    public function login(string $username = null, string $password = null): ?string
    {
        if ($username == null || $password == null) {
            return null;
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'POST',
                'body' => [
                    'email' => $username,
                    'password' => $password
                ]
            ]);
            $response = wp_remote_request($this->apiUrl . self::AUTH_URL, $args);

            return $this->handleResponse($response, true);
        } catch (\Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    private function handleResponse(array $response, bool $plain = false)
    {
        switch ($response['response']['code']) {
            case 200:
                if ($plain) {
                    return $response['body'];
                }
                return json_decode($response['body']);
                break;
            case 403:
                throw new AccessForbiden('Invalid username or password.');
                break;
            case 422:
            case 500:
                $message = $this->getServerErrorMessage($response);
                throw new ServerErrorException($message);
                break;
        }
    }

    private function getServerErrorMessage($response): string
    {
        if (strpos($response['body'], '!DOCTYPE html') !== false) {
            return str_replace('<!-- ', '', explode('-->', $response['body'], 2)[0]);
        }
        $body = json_decode($response['body']);
        if ($body->error) {
            if (strpos($body->message, 'tack trace') !== false) {
                return explode('tack trace', $body->message, 2)[0];
            }
            if (strpos($body->message, '#0 /') !== false) {
                return explode('#0 /', $body->message, 2)[0];
            }
            return $body->message;
        }
        FrisboDebugger::lff($response);
        return 'Unknown error';
    }

    public function getOrganizations(): array
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'GET',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken]
            ]);
            $response = wp_remote_request($this->apiUrl . self::ORGANIZATIONS_URL, $args);

            $organizationsArray = $this->handleResponse($response);
            return array_map(function ($organization) {
                return Organization::fromObject($organization);
            }, $organizationsArray);
        } catch (\Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function getOrganizationChannels(int $organizationId): array
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'GET',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken]
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace('{organizationId}', $organizationId, self::ORGANIZATIONS_CHANNEL_URL), $args);

            $channelsArray = $this->handleResponse($response);
            return array_map(function ($organization) {
                return Channel::fromObject($organization);
            }, $channelsArray);
        } catch (\Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function sendProduct($organizationId, Product $product): Product
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'POST',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
                'body' => (array) $product
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace('{organizationId}', $organizationId, self::ORGANIZATIONS_PRODUCT_URL), $args);

            $productResponse = $this->handleResponse($response);
            return Product::fromObject($productResponse);
        } catch (\Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            if (strpos($e->getMessage(), 'already exists') !== false) {
                throw new ProductExists($e->getMessage());
            }
            throw $e;
        }
    }

    public function sendOrder($organizationId, Order $order): Order
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'POST',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
                'body' => (array) $order
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace('{organizationId}', $organizationId, self::ORGANIZATIONS_ORDERS_URL), $args);

            $orderResponse = $this->handleResponse($response);
            return Order::fromObject($orderResponse);
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            if (strpos($e->getMessage(), 'already exists') !== false) {
                throw new OrderExists($e->getMessage());
            }
            throw $e;
        }
    }

    public function updateOrder($organizationId, Order $order): Order
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'PUT',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
                'body' => (array) $order
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace(['{organizationId}', '{orderId}'], [$organizationId, $order->order_id], self::ORGANIZATIONS_ORDER_URL), $args);

            $orderResponse = $this->handleResponse($response);
            return Order::fromObject($orderResponse);
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function cancelOrder($organizationId, Order $order): Order
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'DELETE',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
                'body' => (array) $order
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace(['{organizationId}', '{orderId}'], [$organizationId, $order->order_id], self::ORGANIZATIONS_ORDER_URL), $args);

            $orderResponse = $this->handleResponse($response);
            return Order::fromObject($orderResponse);
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function transitionOrder($organizationId, Order $order, string $transitionName): ?string
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'POST',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
                'body' => (array) $order
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace(['{organizationId}', '{orderId}', '{transitionName}'], [$organizationId, $order->order_id, $transitionName], self::TRANSITION_ORDER_URL), $args);

            return $this->handleResponse($response, true);
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function getProductsByFilter($organizationId, Filter ...$filters): array
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'GET',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace('{organizationId}', $organizationId, self::ORGANIZATIONS_PRODUCT_URL) . '?' . Filter::combineFilters(...$filters), $args);

            $productResponse = $this->handleResponse($response);
            return array_map(
                function ($product) {
                    return Product::fromObject($product);
                },
                $productResponse->data
            );
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function getOrdersByFilter($organizationId, Filter ...$filters): array
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'GET',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace('{organizationId}', $organizationId, self::ORGANIZATIONS_ORDERS_URL) . '?' . Filter::combineFilters(...$filters), $args);

            $orderResponse = $this->handleResponse($response);
            return array_map(
                function ($order) {
                    return Order::fromObject($order);
                },
                $orderResponse->data
            );
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function getStocksByFilter($organizationId, $channelId, Filter ...$filters): array
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        try {
            $args = array_merge_recursive(self::HTTP_ARGS, [
                'method' => 'GET',
                'headers' => ['Authorization' => 'Bearer ' . $accessToken],
            ]);
            $response = wp_remote_request($this->apiUrl . str_replace(['{organizationId}', '{channelId}'], [$organizationId, $channelId], self::ORGANIZATIONS_CHANNEL_STOCK_URL) . '?' . Filter::combineFilters(...$filters), $args);

            $storageResponse = $this->handleResponse($response);
            return array_map(
                function ($storage) {
                    return Storage::fromObject($storage);
                },
                $storageResponse
            );
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }
}
