<?php

use Frisbo\FrisboSdk\FrisboCache;

require_once FRISBO_PATH . '/app/FrisboEnvs.php';
require_once FRISBO_TESTS . '/WoocommerceTrait.php';

class FrisboWoocommerce extends WP_UnitTestCase
{
    use WoocommerceTrait;

    /**
     * @var Frisbo
     */
    private $frisbo;

    const TIMEOUT = 60;
    const EMAIL = FrisboEnvs::EMAIL_2;
    const PASSWORD = FrisboEnvs::PASSWORD_2;

    public function set_up()
    {
        $this->frisbo = Frisbo::instance();
    }

    public function testPrefix()
    {
        $prefix = '_' . $this->getRand();
        $this->frisbo->settings->orderPrefix = $prefix;
        $this->frisbo->settings->updateSettings();
        $this->assertEquals($prefix, $this->frisbo->settings->orderPrefix);
    }

    public function testFrisboCredentials()
    {
        $this->frisbo->setClient(self::EMAIL, self::PASSWORD);
        $response = json_decode($this->frisbo->client->login(self::EMAIL, self::PASSWORD));
        $this->assertNotNull($response->access_token);

        $this->frisbo->settings->email = self::EMAIL;
        $this->frisbo->settings->password = self::PASSWORD;
        $this->frisbo->settings->updateSettings();

        $this->assertEquals(self::EMAIL, $this->frisbo->settings->email);
        $this->assertEquals(self::PASSWORD, $this->frisbo->settings->password);
    }

    public function testGetOrganizations()
    {
        $organizations = $this->frisbo->client->getOrganizations();
        $this->assertGreaterThan(0, count($organizations));
        $this->frisbo->settings->organizationId = $organizations[0]->organization_id;
        $this->frisbo->settings->updateSettings();
    }

    public function testGetChannels()
    {
        $channels = $this->frisbo->client->getOrganizationChannels($this->frisbo->settings->organizationId);
        $this->assertGreaterThan(0, count($channels));
        $this->frisbo->settings->channelId = $channels[0]->id;
        $this->frisbo->settings->updateSettings();
    }

    public function testSyncProduct()
    {
        $product = $this->createProduct();
        $frisboProducts = new FrisboProducts();
        $frisboProducts->sync_products();

        $frisboProduct = $this->waitFrisboProduct($product->get_sku());
        $product = $this->getProduct($product->get_id());
        $this->assertEquals($frisboProduct->product_id, $product->get_meta('frisbo_product_id'));
        return $product;
    }

    public function testSyncProductEan()
    {
        $ean = '123456789';
        $this->frisbo->settings->eanFields = 'ean_field1, ean_field2';
        $this->frisbo->settings->updateSettings();

        $product = $this->createProduct();

        $product->update_meta_data('ean_field2', $ean);
        $product->save_meta_data();

        $frisboProducts = new FrisboProducts();
        $frisboProducts->sync_products();

        $frisboProduct = $this->waitFrisboProduct($product->get_sku());
        $this->assertEquals($frisboProduct->ean, $ean);
    }

    /**
     * IMPORTANT: This test will pass only if user is superadmin
     * 
     * @depends testSyncProduct
     */
    public function testSyncStock(WC_Product $product)
    {
        $this->frisbo->settings->overwriteStock = 'on';
        $this->frisbo->settings->updateSettings();


        $frisboProductId = $product->get_meta('frisbo_product_id');
        $warehouses = $this->frisbo->client->getOrganizationWarehouses($this->frisbo->settings->organizationId);

        $quantity = $this->getRand(true);
        $this->frisbo->client->supplyStock($frisboProductId, [
            'warehouse_id' => $warehouses[0]->id,
            'quantity' => $quantity
        ]);

        $frisboProducts = new FrisboProducts();
        $frisboProducts->sync_stock();

        $product = $this->getProduct($product->get_id());
        $this->assertEquals($quantity, $product->get_stock_quantity());
        return $product;
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrder(WC_Product $product = null)
    {
        $order = $this->createOrder([
            'products' => [$product]
        ]);
        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $order = $this->getOrder($order->get_id());
        $this->assertEquals($frisboOrder->order_id, $order->get_meta('frisbo_order_number'));
        return $order;
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderWithStatus(WC_Product $product = null)
    {
        $status = 'on-hold';
        $this->frisbo->settings->orderStatusSync = $status;
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);
        $order->update_status($status);
        $this->assertEquals($status, $order->get_status());
        $order->save();

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $order = $this->getOrder($order->get_id());
        $this->assertEquals($frisboOrder->order_id, $order->get_meta('frisbo_order_number'));
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderWithStatusError(WC_Product $product = null)
    {
        $status = 'on-hold';
        $statusSettings = "$status,bacs:processing";
        $this->frisbo->settings->orderStatusSync = $statusSettings;
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $order = $this->getOrder($order->get_id());
        $this->assertEquals("To be syncronized order has to be in status $status", $order->get_meta('reason_status'));
    }


    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderWithStatusAndPayment(WC_Product $product = null)
    {
        $paymentMethod = 'bacs';
        $status = 'processing';
        $statusSetting = "on-hold, $paymentMethod:$status";
        $this->frisbo->settings->orderStatusSync = $statusSetting;
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);
        $order->set_payment_method($paymentMethod);
        $this->assertEquals($paymentMethod, $order->get_payment_method());
        $order->update_status($status);
        $this->assertEquals($status, $order->get_status());
        $order->save();

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $order = $this->getOrder($order->get_id());
        $this->assertEquals($frisboOrder->order_id, $order->get_meta('frisbo_order_number'));
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderWithStatusAndPaymentError(WC_Product $product = null)
    {
        $paymentMethod = 'bacs';
        $status = 'on-hold';
        $statusSetting = "processing, $paymentMethod:$status";
        $this->frisbo->settings->orderStatusSync = $statusSetting;
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);
        $order->set_payment_method($paymentMethod);
        $order->save();

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();
        $order = $this->getOrder($order->get_id());

        $this->assertEquals("Order with payment method $paymentMethod has to be in status $status to be syncronized.", $order->get_meta('reason_status'));
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderVatRegNumber(WC_Product $product = null)
    {
        $vatNumber = '987654321';
        $this->frisbo->settings->orderStatusSync = '';
        $this->frisbo->settings->vatRegNumberFields = 'vat_reg_number_field1, vat_reg_number_field2, vat_reg_number_field3';
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);

        $order->update_meta_data('vat_reg_number_field3', $vatNumber);
        $order->save_meta_data();

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $this->assertEquals($frisboOrder->billing_customer->vat_registration_number, $vatNumber);
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderPackingNote(WC_Product $product = null)
    {
        $note = 'Leave it in front of the door.';
        $this->frisbo->settings->orderStatusSync = '';
        $this->frisbo->settings->packingNoteFields = 'packing_notes1, packing_notes2, packing_notes3';
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);

        $order->update_meta_data('packing_notes2', $note);
        $order->save_meta_data();

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $this->assertEquals($note, $frisboOrder->packing_notes);
    }

    /**
     * @depends testSyncProduct
     */
    public function testSyncOrderLocker(WC_Product $product = null)
    {
        $lockerId = '83231';
        $lockerSetting = 'DPD@dpd_locker_id';
        $this->frisbo->settings->lockerFields = "FAN@fan_locker_id, Cargus@cargus_locker_id, $lockerSetting";
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);

        $order->update_meta_data('dpd_locker_id', $lockerId);
        $order->save_meta_data();

        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $this->assertEquals($frisboOrder->shipping_address->locker, "DPD@$lockerId");
    }

    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderDefaultCurrency(WC_Product $product = null)
    {
        $currency = 'USD';

        $order = $this->createOrder([
            'products' => [$product]
        ]);

        $frisboOrderMappings = (new FrisboOrderDto($order))->getMappings();
        $this->assertEquals($frisboOrderMappings->metadata['payment_currency'], $currency);
    }


    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderCurrency(WC_Product $product = null)
    {
        $currency = 'EUR';
        $this->frisbo->settings->currencyFields = "currency_field_1,currency_field_2,currency_field_3";
        $this->frisbo->settings->updateSettings();

        $order = $this->createOrder([
            'products' => [$product]
        ]);

        $order->update_meta_data('currency_field_2', $currency);
        $order->save_meta_data();

        $frisboOrderMappings = (new FrisboOrderDto($order))->getMappings();
        $this->assertEquals($frisboOrderMappings->metadata['payment_currency'], $currency);
    }

    /** 
     * @depends testSyncOrder
     */
    public function testSyncOrderStatus(WC_Order $order = null)
    {
        $frisboOrder = $this->waitFrisboOrder($order->get_order_number(), ['invoiced']);
        $this->frisbo->client->transitionOrder($this->frisbo->settings->organizationId, $frisboOrder, 'shipOrder');
        $frisboOrder = $this->waitFrisboOrder($order->get_order_number(), ['shipped']);
        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders_status();

        $order = $this->getOrder($order->get_id());
        $this->assertEquals($frisboOrder->shipping_tracking_number, $order->get_meta('frisbo_awb_number'));
        $this->assertEquals($frisboOrder->tracking_url, $order->get_meta('frisbo_awb_tracking_url'));
        $this->assertEquals($frisboOrder->shipped_with, $order->get_meta('frisbo_shipped_with'));
        $this->assertEquals(FrisboOrders::getAttachmentUrl($frisboOrder->organization_id, $frisboOrder->order_id), $order->get_meta('frisbo_awb_label_url'));
        $this->assertEquals(FrisboOrders::getAttachmentUrl($frisboOrder->organization_id, $frisboOrder->order_id, 'invoice'), $order->get_meta('frisbo_invoice_url'));
    }


    /** 
     * @depends testSyncProduct
     */
    public function testSyncOrderWithVirtualProduct(WC_Product $product)
    {
        $product1 = $this->createProduct([
            'isVirtual' => true
        ]);
        $order = $this->createOrder([
            'products' => [$product1, $product]
        ]);
        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();

        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $frisboOrderVirtualProduct = array_filter($frisboOrder->products, function ($product) {
            return $product->is_virtual ?? false;
        })[0];
        $this->assertEmpty($frisboOrderVirtualProduct->sku);
        $this->assertEquals(1, $frisboOrderVirtualProduct->is_virtual);
    }


    /** 
     * @depends testSyncProduct
     */
    public function testSyncCompanyOrder(WC_Product $product)
    {
        $companyName = 'FRISBO SA';
        $order = $this->createOrder([
            'company' => $companyName,
            'products' => [$product],
        ]);
        $frisboOrders = new FrisboOrders();
        $frisboOrders->sync_orders();
        $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
        $this->assertEquals($companyName, $frisboOrder->billing_customer->first_name);
    }
}
